

from shadow4.beamline.optical_elements.gratings.s4_plane_grating import S4PlaneGrating, S4PlaneGratingElement
from shadow4.beamline.optical_elements.gratings.s4_sphere_grating import S4SphereGrating, S4SphereGratingElement
from shadow4.beamline.s4_optical_element_decorators import SurfaceCalculation, S4SphereOpticalElementDecorator


import numpy
import sys
import xraylib

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui

from shadow4.beamline.optical_elements.gratings.s4_plane_grating import S4PlaneGrating, S4PlaneGratingElement
from shadow4.beamline.optical_elements.gratings.s4_sphere_grating import S4SphereGrating, S4SphereGratingElement
from shadow4.beamline.optical_elements.gratings.s4_conic_grating import S4ConicGrating, S4ConicGratingElement
from shadow4.beamline.optical_elements.gratings.s4_toroid_grating import S4ToroidGrating, S4ToroidGratingElement
from shadow4.beamline.optical_elements.gratings.s4_numerical_mesh_grating import S4NumericalMeshGrating, S4NumericalMeshGratingElement
from shadow4.beamline.optical_elements.gratings.s4_additional_numerical_mesh_grating import S4AdditionalNumericalMeshGrating, S4AdditionalNumericalMeshGratingElement


from orangecontrib.shadow4.util.shadow4_objects import ShadowData
from orangecontrib.shadow4.widgets.gui.ow_optical_element_with_surface_shape import OWOpticalElementWithSurfaceShape

from orangecontrib.shadow4.util.shadow4_objects import VlsPgmPreProcessorData
import copy


class _OWGrating(OWOpticalElementWithSurfaceShape):
    # name = "Generic Grating"
    # description = "Shadow Grating"
    # icon = "icons/plane_grating.png"
    #
    # priority = 1.390
    #
    # inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
    # inputs.append(("VLS-PGM PreProcessor Data", VlsPgmPreProcessorData, "setVlsPgmPreProcessorData"))
    #
    # def get_oe_type(self):
    #     return "grating", "Grating"

    #########################################################
    # grating
    #########################################################

    ruling                 = Setting(800e3)
    ruling_coeff_linear    = Setting(0.0)
    ruling_coeff_quadratic = Setting(0.0)
    ruling_coeff_cubic     = Setting(0.0)
    ruling_coeff_quartic   = Setting(0.0)
    order = Setting(-1)
    f_ruling = Setting(1)
    # file_refl = Setting("")

    def __init__(self):
        super(_OWGrating, self).__init__()
        # with gratings no "internal surface parameters" allowed. Fix value and hide selecting combo:
        self.surface_shape_parameters = 1
        self.surface_shape_internal_external_box.setVisible(False)

    def create_basic_settings_specific_subtabs(self, tabs_basic_setting):
        subtab_grating_diffraction = oasysgui.createTabPage(tabs_basic_setting, "Grating")    # to be populated
        subtab_grating_efficiency = oasysgui.createTabPage(tabs_basic_setting, "G. Efficiency")    # to be populated

        return subtab_grating_diffraction, subtab_grating_efficiency

    def populate_basic_settings_specific_subtabs(self, specific_subtabs):
        subtab_grating_diffraction, subtab_grating_efficiency = specific_subtabs

        #########################################################
        # Basic Settings / Grating Diffraction
        #########################################################
        self.populate_tab_grating_diffraction(subtab_grating_diffraction)

        #########################################################
        # Basic Settings / Grating Efficiency
        #########################################################
        self.populate_tab_grating_efficiency(subtab_grating_efficiency)

    def populate_tab_grating_diffraction(self, subtab_grating_diffraction):

        grating_box = oasysgui.widgetBox(subtab_grating_diffraction, "Grating Diffraction", addSpace=True, orientation="vertical")


        gui.comboBox(grating_box, self, "f_ruling", tooltip="f_ruling",
                     label="Ruling type", labelWidth=120,
                     items=["Constant on X-Y plane",
                            "VLS Variable (Polynomial) Line Density"],
                     sendSelectedValue=False, orientation="horizontal",
                     callback=self.grating_diffraction_tab_visibility)

        gui.separator(grating_box)

        oasysgui.lineEdit(grating_box, self, "ruling", tooltip="ruling",
                          label="ruling (coeff 0; lines/m)", addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")

        self.grating_box_vls = oasysgui.widgetBox(grating_box, "", addSpace=False, orientation="vertical")

        oasysgui.lineEdit(self.grating_box_vls, self, "ruling_coeff_linear", tooltip="ruling_coeff_linear",
                          label="ruling (coeff 1; Lines/m\u00b2])", addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")

        oasysgui.lineEdit(self.grating_box_vls, self, "ruling_coeff_quadratic", tooltip="ruling_coeff_quadratic",
                          label="ruling (coeff 2; Lines/m\u00b3])", addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")

        oasysgui.lineEdit(self.grating_box_vls, self, "ruling_coeff_cubic", tooltip="ruling_coeff_cubic",
                          label="ruling (coeff 3; Lines/m\u2074])", addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")

        oasysgui.lineEdit(self.grating_box_vls, self, "ruling_coeff_quartic", tooltip="ruling_coeff_quartic",
                          label="ruling (coeff 4; Lines/m\u2075])", addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")

        oasysgui.lineEdit(grating_box, self, "order", tooltip="order",
                          label="Diffraction order (- for inside orders)", addSpace=True,
                          valueType=int, labelWidth=200, orientation="horizontal")

        #
        #
        # ## preprocessor file
        # self.crystal_box_1 = oasysgui.widgetBox(crystal_box, "", addSpace=False, orientation="vertical")
        #
        # file_box = oasysgui.widgetBox(self.crystal_box_1, "", addSpace=False, orientation="horizontal", height=30)
        #
        # self.le_file_crystal_parameters = oasysgui.lineEdit(file_box, self, "file_crystal_parameters",
        #                                                     "File (preprocessor)", tooltip="file_crystal_parameters",
        #                                                     labelWidth=150, valueType=str, orientation="horizontal")
        #
        # gui.button(file_box, self, "...", callback=self.select_file_crystal_parameters)
        #
        # ## xoppy file
        # self.crystal_box_2 = oasysgui.widgetBox(crystal_box, "", addSpace=False, orientation="vertical")
        #
        #
        # crystal_box_2_1 = oasysgui.widgetBox(self.crystal_box_2, "", addSpace=False, orientation="horizontal")
        #
        # self.le_file_diffraction_profile = oasysgui.lineEdit(crystal_box_2_1, self,
        #                                                      "file_diffraction_profile", "File (user Diff Profile)",
        #                                                      tooltip="file_diffraction_profile",
        #                                                      labelWidth=120, valueType=str, orientation="horizontal")
        # gui.button(crystal_box_2_1, self, "...", callback=self.select_file_diffraction_profile)
        #
        # oasysgui.lineEdit(self.crystal_box_2, self, "user_defined_bragg_angle",
        #                   "Bragg Angle respect to the surface [deg]", tooltip="user_defined_bragg_angle",
        #                   labelWidth=260, valueType=float,
        #                   orientation="horizontal", callback=self.crystal_diffraction_tab_visibility)
        # oasysgui.lineEdit(self.crystal_box_2, self, "user_defined_asymmetry_angle", "Asymmetry angle [deg]",
        #                   tooltip="user_defined_asymmetry_angle",
        #                   labelWidth=260, valueType=float, orientation="horizontal",
        #                   callback=self.crystal_diffraction_tab_visibility)
        #
        # ##  parameters for internal calculations / xoppy file
        # self.crystal_box_3 = oasysgui.widgetBox(crystal_box, "", addSpace=False, orientation="vertical") #, height=340)
        #
        # gui.comboBox(self.crystal_box_3, self, "user_defined_crystal", tooltip="user_defined_crystal",
        #              label="Crystal", addSpace=True,
        #              items=self.CRYSTALS, sendSelectedValue=False, orientation="horizontal", labelWidth=260)
        #
        # box_miller = oasysgui.widgetBox(self.crystal_box_3, "", orientation="horizontal", width=350)
        # oasysgui.lineEdit(box_miller, self, "user_defined_h", tooltip="user_defined_h",
        #                   label="Miller Indices [h k l]", addSpace=True,
        #                   valueType=int, labelWidth=200, orientation="horizontal")
        # oasysgui.lineEdit(box_miller, self, "user_defined_k", tooltip="user_defined_k",
        #                   addSpace=True, valueType=int, orientation="horizontal")
        # oasysgui.lineEdit(box_miller, self, "user_defined_l", tooltip="user_defined_l",
        #                   addSpace=True, valueType=int, orientation="horizontal")
        #
        #
        # ## autosetting
        # self.crystal_box_4 = oasysgui.widgetBox(crystal_box, "", addSpace=False, orientation="vertical") #, height=240)
        #
        # gui.comboBox(self.crystal_box_4, self, "crystal_auto_setting", tooltip="crystal_auto_setting",
        #              label="Auto setting", labelWidth=350, items=["No", "Yes"],
        #              callback=self.crystal_diffraction_tab_visibility, sendSelectedValue=False, orientation="horizontal")
        #
        # gui.separator(self.crystal_box_4, height=10)
        #
        # ##
        # self.autosetting_box = oasysgui.widgetBox(self.crystal_box_4, "", addSpace=False,
        #                                           orientation="vertical")
        # self.autosetting_box_empty = oasysgui.widgetBox(self.crystal_box_4, "", addSpace=False,
        #                                                 orientation="vertical")
        #
        # self.autosetting_box_units = oasysgui.widgetBox(self.autosetting_box, "", addSpace=False, orientation="vertical")
        #
        # gui.comboBox(self.autosetting_box_units, self, "units_in_use", tooltip="units_in_use", label="Units in use",
        #              labelWidth=260, items=["eV", "Angstroms"],
        #              callback=self.crystal_diffraction_tab_visibility, sendSelectedValue=False, orientation="horizontal")
        #
        # self.autosetting_box_units_1 = oasysgui.widgetBox(self.autosetting_box_units, "", addSpace=False,
        #                                                   orientation="vertical")
        #
        # oasysgui.lineEdit(self.autosetting_box_units_1, self, "photon_energy", "Set photon energy [eV]",
        #                   tooltip="photon_energy", labelWidth=260,
        #                   valueType=float, orientation="horizontal")
        #
        # self.autosetting_box_units_2 = oasysgui.widgetBox(self.autosetting_box_units, "", addSpace=False,
        #                                                   orientation="vertical")
        #
        # oasysgui.lineEdit(self.autosetting_box_units_2, self, "photon_wavelength", "Set wavelength [Å]",
        #                   tooltip="photon_wavelength", labelWidth=260,
        #                   valueType=float, orientation="horizontal")
        #
        #
        self.grating_diffraction_tab_visibility()

    def populate_tab_grating_efficiency(self, subtab_grating_efficiency):
        pass

        # self.asymmetric_cut_box = oasysgui.widgetBox(subtab_crystal_geometry, "", addSpace=False, orientation="vertical",
        #                                              height=110)
        #
        # self.asymmetric_cut_combo = gui.comboBox(self.asymmetric_cut_box, self, "asymmetric_cut",
        #                                          tooltip="asymmetric_cut", label="Asymmetric cut",
        #                                          labelWidth=355,
        #                                          items=["No", "Yes"],
        #                                          callback=self.crystal_geometry_tab_visibility, sendSelectedValue=False,
        #                                          orientation="horizontal")
        #
        # self.asymmetric_cut_box_1 = oasysgui.widgetBox(self.asymmetric_cut_box, "", addSpace=False, orientation="vertical")
        # self.asymmetric_cut_box_1_empty = oasysgui.widgetBox(self.asymmetric_cut_box, "", addSpace=False,
        #                                                      orientation="vertical")
        #
        # oasysgui.lineEdit(self.asymmetric_cut_box_1, self, "planes_angle", "Planes angle [deg]",
        #                   tooltip="planes_angle", labelWidth=260,
        #                   valueType=float, orientation="horizontal")
        #
        # self.asymmetric_cut_box_1_order = oasysgui.widgetBox(self.asymmetric_cut_box_1, "", addSpace=False,
        #                                                      orientation="vertical")
        #
        # # oasysgui.lineEdit(self.asymmetric_cut_box_1_order, self,
        # #                   "below_onto_bragg_planes", "Below[-1]/onto[1] bragg planes **deleted**",
        # #                   tooltip="below_onto_bragg_planes",
        # #                   labelWidth=260, valueType=float, orientation="horizontal")
        #
        # self.thickness_box = oasysgui.widgetBox(subtab_crystal_geometry, "", addSpace=False, orientation="vertical",
        #                                              height=110)
        #
        # self.thickness_combo = gui.comboBox(self.thickness_box, self, "is_thick",
        #                                          tooltip="is_thick", label="Thick crystal approx.",
        #                                          labelWidth=355,
        #                                          items=["No", "Yes"],
        #                                          callback=self.crystal_geometry_tab_visibility, sendSelectedValue=False,
        #                                          orientation="horizontal")
        #
        # self.thickness_box_1 = oasysgui.widgetBox(self.thickness_box, "", addSpace=False, orientation="vertical")
        # self.thickness_box_1_empty = oasysgui.widgetBox(self.thickness_box_1, "", addSpace=False,
        #                                                      orientation="vertical")
        #
        # self.le_thickness_1 = oasysgui.lineEdit(self.thickness_box_1, self,
        #                                         "thickness", "Crystal thickness [m]",tooltip="thickness",
        #                                         valueType=float, labelWidth=260, orientation="horizontal")
        #
        # # self.set_BraggLaue()
        #
        # # gui.separator(self.mosaic_box_1)
        #
        # # self.johansson_box = oasysgui.widgetBox(self.mosaic_box_1, "", addSpace=False, orientation="vertical", height=100)
        # #
        # # gui.comboBox(self.johansson_box, self, "johansson_geometry", tooltip="johansson_geometry",
        # #              label="Johansson Geometry **deleted**", labelWidth=355, items=["No", "Yes"],
        # #              callback=self.crystal_geometry_tab_visibility, sendSelectedValue=False, orientation="horizontal")
        # #
        # # self.johansson_box_1 = oasysgui.widgetBox(self.johansson_box, "", addSpace=False, orientation="vertical")
        # # self.johansson_box_1_empty = oasysgui.widgetBox(self.johansson_box, "", addSpace=False, orientation="vertical")
        # #
        # # self.le_johansson_radius = oasysgui.lineEdit(self.johansson_box_1, self, "johansson_radius", "Johansson radius",
        # #                                              tooltip="johansson_radius",
        # #                                              labelWidth=260, valueType=float, orientation="horizontal")
        # #
        # # self.mosaic_box_2 = oasysgui.widgetBox(mosaic_box, "", addSpace=False, orientation="vertical")
        # #
        # # oasysgui.lineEdit(self.mosaic_box_2, self, "angle_spread_FWHM", "Angle spread FWHM [deg]",
        # #                   tooltip="angle_spread_FWHM", labelWidth=260,
        # #                   valueType=float, orientation="horizontal")
        # # self.le_thickness_2 = oasysgui.lineEdit(self.mosaic_box_2, self, "thickness", "Thickness",
        # #                                         tooltip="thickness", labelWidth=260,
        # #                                         valueType=float, orientation="horizontal")
        # # oasysgui.lineEdit(self.mosaic_box_2, self, "seed_for_mosaic", "Seed for mosaic [>10^5]",
        # #                   tooltip="seed_for_mosaic", labelWidth=260,
        # #                   valueType=float, orientation="horizontal")
        #
        # # self.set_Mosaic()
        #
        # self.crystal_geometry_tab_visibility()

    #########################################################
    # Grating Methods
    #########################################################

    def setVlsPgmPreProcessorData(self, data):
        if data is not None:
            self.surface_shape_type = 0
            self.surface_shape_tab_visibility()

            self.source_plane_distance = data.d_mirror_to_grating/2 * 1e-3
            self.image_plane_distance = data.d_grating_to_exit_slits * 1e-3

            self.angles_respect_to = 0
            self.incidence_angle_deg = data.alpha
            self.reflection_angle_deg =data.beta
            self.calculate_incidence_angle_mrad()
            self.calculate_reflection_angle_mrad()

            self.oe_orientation_angle = 2
            self.order = -1

            self.f_ruling = 1
            self.ruling = data.shadow_coeff_0 * 1e3
            self.ruling_coeff_linear = data.shadow_coeff_1 * 1e6
            self.ruling_coeff_quadratic = data.shadow_coeff_2 * 1e9
            self.ruling_coeff_cubic = data.shadow_coeff_3 * 1e12
            self.ruling_coeff_quartic = 0.0
            self.grating_diffraction_tab_visibility()


    def grating_diffraction_tab_visibility(self):
        self.grating_box_vls.setVisible(self.f_ruling==1)

    #
    # def crystal_geometry_tab_visibility(self):
    #     # self.set_mosaic()
    #     self.set_asymmetric_cut()
    #     self.set_thickness()
    #     # self.set_johansson_geometry()
    #
    # def set_diffraction_calculation(self):
    #     self.crystal_box_1.setVisible(False)
    #     self.crystal_box_2.setVisible(False)
    #     self.crystal_box_3.setVisible(False)
    #
    #     if (self.diffraction_calculation == 0):   # internal xraylib
    #         self.crystal_box_3.setVisible(True)
    #     elif (self.diffraction_calculation == 1): # internal
    #         self.crystal_box_3.setVisible(True)
    #     elif (self.diffraction_calculation == 2): # preprocessor bragg v1
    #         self.crystal_box_1.setVisible(True)
    #     elif (self.diffraction_calculation == 3): # preprocessor bragg v2
    #         self.crystal_box_1.setVisible(True)
    #     elif (self.diffraction_calculation == 4): # user file, E-independent
    #         self.crystal_box_2.setVisible(True)
    #     elif (self.diffraction_calculation == 5): # user file, E-dependent
    #         self.crystal_box_2.setVisible(True)
    #
    #     if self.diffraction_calculation in (4,5):
    #         self.incidence_angle_deg_le.setEnabled(True)
    #         self.incidence_angle_rad_le.setEnabled(True)
    #         self.reflection_angle_deg_le.setEnabled(True)
    #         self.reflection_angle_rad_le.setEnabled(True)
    #
    # def select_file_crystal_parameters(self):
    #     self.le_file_crystal_parameters.setText(oasysgui.selectFileFromDialog(self, self.file_crystal_parameters, "Select File With Crystal Parameters"))
    #
    # def set_autosetting(self):
    #     self.autosetting_box_empty.setVisible(self.crystal_auto_setting == 0)
    #     self.autosetting_box.setVisible(self.crystal_auto_setting == 1)
    #
    #     if self.crystal_auto_setting == 0:
    #         self.incidence_angle_deg_le.setEnabled(True)
    #         self.incidence_angle_rad_le.setEnabled(True)
    #         self.reflection_angle_deg_le.setEnabled(True)
    #         self.reflection_angle_rad_le.setEnabled(True)
    #     else:
    #         self.incidence_angle_deg_le.setEnabled(False)
    #         self.incidence_angle_rad_le.setEnabled(False)
    #         self.reflection_angle_deg_le.setEnabled(False)
    #         self.reflection_angle_rad_le.setEnabled(False)
    #         self.set_units_in_use()
    #
    # def set_units_in_use(self):
    #     self.autosetting_box_units_1.setVisible(self.units_in_use == 0)
    #     self.autosetting_box_units_2.setVisible(self.units_in_use == 1)
    #
    # def select_file_diffraction_profile(self):
    #     self.le_file_diffraction_profile.setText(oasysgui.selectFileFromDialog(self, self.file_diffraction_profile, "Select File With User Defined Diffraction Profile"))
    #
    #
    # def set_asymmetric_cut(self):
    #     self.asymmetric_cut_box_1.setVisible(self.asymmetric_cut == 1)
    #     self.asymmetric_cut_box_1_empty.setVisible(self.asymmetric_cut == 0)
    #
    # def set_thickness(self):
    #     self.thickness_box_1.setVisible(self.is_thick == 0)
    #     self.thickness_box_1_empty.setVisible(self.is_thick == 1)
    #


    #########################################################
    # S4 objects
    #########################################################

    def get_optical_element_instance(self):

        if self.surface_shape_type > 0 and self.surface_shape_parameters == 0:
            raise ValueError("Curved grating with internal calculation not allowed.")


        if self.surface_shape_type == 0:
            grating = S4PlaneGrating(
                name="Plane Grating",
                boundary_shape=self.get_boundary_shape(),
                ruling=self.ruling,
                ruling_coeff_linear=self.ruling_coeff_linear,
                ruling_coeff_quadratic=self.ruling_coeff_quadratic,
                ruling_coeff_cubic=self.ruling_coeff_cubic,
                ruling_coeff_quartic=self.ruling_coeff_quartic,
                coating=None,
                coating_thickness=None,
                order=self.order,
                f_ruling=self.f_ruling,
            )

        elif self.surface_shape_type == 1:
            print("FOCUSING DISTANCES: convexity:  ", numpy.logical_not(self.surface_curvature).astype(int))
            print("FOCUSING DISTANCES: radius:  ", self.spherical_radius)

            grating = S4SphereGrating(
                name="Sphere Grating",
                boundary_shape=self.get_boundary_shape(),
                ruling=self.ruling,
                ruling_coeff_linear=self.ruling_coeff_linear,
                ruling_coeff_quadratic=self.ruling_coeff_quadratic,
                ruling_coeff_cubic=self.ruling_coeff_cubic,
                ruling_coeff_quartic=self.ruling_coeff_quartic,
                coating=None,
                coating_thickness=None,
                order=self.order,
                f_ruling=self.f_ruling,
                #
                # surface_calculation=SurfaceCalculation.EXTERNAL,
                radius=self.spherical_radius,
                is_cylinder=self.is_cylinder,
                cylinder_direction=self.cylinder_orientation, #  Direction:  TANGENTIAL = 0  SAGITTAL = 1
                convexity=numpy.logical_not(self.surface_curvature).astype(int), #  Convexity: NONE = -1  UPWARD = 0  DOWNWARD = 1
            )
        elif self.surface_shape_type == 5:
            grating = S4ToroidGrating(
                name="Toroid Grating",
                boundary_shape=self.get_boundary_shape(),
                ruling=self.ruling,
                ruling_coeff_linear=self.ruling_coeff_linear,
                ruling_coeff_quadratic=self.ruling_coeff_quadratic,
                ruling_coeff_cubic=self.ruling_coeff_cubic,
                ruling_coeff_quartic=self.ruling_coeff_quartic,
                coating=None,
                coating_thickness=None,
                order=self.order,
                f_ruling=self.f_ruling,
                #
                min_radius=self.torus_minor_radius,
                maj_radius=self.torus_major_radius,
                f_torus=self.toroidal_mirror_pole_location,
            )
        elif self.surface_shape_type == 6:
            grating = S4ConicGrating(
                name="Conic Grating",
                boundary_shape=self.get_boundary_shape(),
                ruling=self.ruling,
                ruling_coeff_linear=self.ruling_coeff_linear,
                ruling_coeff_quadratic=self.ruling_coeff_quadratic,
                ruling_coeff_cubic=self.ruling_coeff_cubic,
                ruling_coeff_quartic=self.ruling_coeff_quartic,
                coating=None,
                coating_thickness=None,
                order=self.order,
                f_ruling=self.f_ruling,
                conic_coefficients=[
                    self.conic_coefficient_0, self.conic_coefficient_1, self.conic_coefficient_2,
                    self.conic_coefficient_3, self.conic_coefficient_4, self.conic_coefficient_5,
                    self.conic_coefficient_6, self.conic_coefficient_7, self.conic_coefficient_8,
                    self.conic_coefficient_9],
            )
        else:
            raise NotImplementedError("surface_shape_type=%d not implemented " % self.urface_shape_type)
        #     crystal = S4ConicCrystal(
        #         name="Conic Crystal",
        #         boundary_shape=self.get_boundary_shape(),
        #         material=self.CRYSTALS[self.user_defined_crystal],
        #         miller_index_h=self.user_defined_h,
        #         miller_index_k=self.user_defined_k,
        #         miller_index_l=self.user_defined_l,
        #         asymmetry_angle=0.0 if not self.asymmetric_cut else numpy.radians(self.planes_angle),
        #         is_thick=self.is_thick,
        #         thickness=self.thickness,
        #         f_central=self.crystal_auto_setting,
        #         f_phot_cent=self.units_in_use,
        #         phot_cent=(self.photon_energy if (self.units_in_use == 0) else self.photon_wavelength),
        #         file_refl=self.file_crystal_parameters,
        #         f_bragg_a=True if self.asymmetric_cut else False,
        #         f_ext=0,
        #         material_constants_library_flag=self.diffraction_calculation,
        #         conic_coefficients=[
        #              self.conic_coefficient_0,self.conic_coefficient_1,self.conic_coefficient_2,
        #              self.conic_coefficient_3,self.conic_coefficient_4,self.conic_coefficient_5,
        #              self.conic_coefficient_6,self.conic_coefficient_7,self.conic_coefficient_8,
        #              self.conic_coefficient_9],
        #     )
        #
        # # if error is selected...

        if self.modified_surface:
            return S4AdditionalNumericalMeshGrating(name="ideal + error Grating",
                        ideal_grating=grating,
                        numerical_mesh_grating=S4NumericalMeshGrating(
                            surface_data_file=self.ms_defect_file_name,
                            boundary_shape=None,
                            name="Sphere Grating",
                            # boundary_shape=self.get_boundary_shape(),
                            ruling=self.ruling,
                            ruling_coeff_linear=self.ruling_coeff_linear,
                            ruling_coeff_quadratic=self.ruling_coeff_quadratic,
                            ruling_coeff_cubic=self.ruling_coeff_cubic,
                            ruling_coeff_quartic=self.ruling_coeff_quartic,
                            coating=None,
                            coating_thickness=None,
                            order=self.order,
                            f_ruling=self.f_ruling,
                            )
                        )
        else:
            return grating



    def get_beamline_element_instance(self):

        if self.modified_surface:
            return S4AdditionalNumericalMeshGratingElement()
        else:
            if self.surface_shape_type == 0:   return  S4PlaneGratingElement()
            elif self.surface_shape_type == 1: return S4SphereGratingElement()
            # elif self.surface_shape_type == 2: return S4EllipsoidCrystalElement()
            # elif self.surface_shape_type == 3: return S4HyperboloidCrystalElement()
            # elif self.surface_shape_type == 4: return S4ParaboloidCrystalElement()
            elif self.surface_shape_type == 5: return S4ToroidGratingElement()
            elif self.surface_shape_type == 6: return S4ConicGratingElement()
            else: raise NotImplementedError("surface_shape_type not yet implemented!")


class OWGrating(_OWGrating):
    name = "Generic Grating"
    description = "Shadow Grating"
    icon = "icons/plane_grating.png"

    priority = 1.390

    inputs = copy.deepcopy(OWOpticalElementWithSurfaceShape.inputs)
    inputs.append(("VLS-PGM PreProcessor Data", VlsPgmPreProcessorData, "setVlsPgmPreProcessorData"))

    def get_oe_type(self):
        return "grating", "Grating"

if __name__ == "__main__":
    from shadow4.beamline.s4_beamline import S4Beamline
    from shadow4.sources.source_geometrical.source_geometrical import SourceGeometrical
    def get_test_beam():
        from shadow4.sources.source_geometrical.source_geometrical import SourceGeometrical
        light_source = SourceGeometrical(name='SourceGeometrical', nrays=5000, seed=5676561)
        light_source.set_spatial_type_point()
        light_source.set_angular_distribution_flat(hdiv1=-0.000000, hdiv2=0.000000, vdiv1=-0.000000, vdiv2=0.000000)
        light_source.set_energy_distribution_uniform(value_min=7990.000000, value_max=8010.000000, unit='eV')
        light_source.set_polarization(polarization_degree=1.000000, phase_diff=0.000000, coherent_beam=0)
        beam = light_source.get_beam()
        return ShadowData(beam=beam, beamline=S4Beamline(light_source=light_source))

    from PyQt5.QtWidgets import QApplication
    a = QApplication(sys.argv)
    ow = OWGrating()
    ow.set_shadow_data(get_test_beam())
    ow.show()
    a.exec_()
    ow.saveSettings()




import os, sys
import numpy

from PyQt5.QtWidgets import QLabel, QApplication, QMessageBox, QSizePolicy
from PyQt5.QtGui import QTextCursor, QIntValidator, QDoubleValidator, QPixmap
from PyQt5.QtCore import Qt

import orangecanvas.resources as resources

from orangewidget import gui, widget
from orangewidget.settings import Setting

from oasys.widgets.widget import OWWidget
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence
from oasys.util.oasys_util import EmittingStream

from orangecontrib.shadow4.util.shadow4_objects import BraggPreProcessorData

from xoppylib.decorators.dabax_decorated import DabaxDecorated
from xoppylib.crystals.create_bragg_preprocessor_file_v2 import create_bragg_preprocessor_file_v2

from urllib.error import HTTPError
from orangecontrib.shadow4.widgets.gui.plots import plot_data1D, plot_multi_data1D

from crystalpy.diffraction.GeometryType import BraggDiffraction
from crystalpy.diffraction.DiffractionSetupShadowPreprocessorV1 import DiffractionSetupShadowPreprocessorV1
from crystalpy.diffraction.DiffractionSetupShadowPreprocessorV2 import DiffractionSetupShadowPreprocessorV2
from crystalpy.diffraction.Diffraction import Diffraction
from crystalpy.util.ComplexAmplitudePhotonBunch import ComplexAmplitudePhotonBunch
from crystalpy.util.ComplexAmplitudePhoton import ComplexAmplitudePhoton
from crystalpy.util.Vector import Vector
from crystalpy.util.Photon import Photon

class OWBragg(OWWidget):
    name = "Bragg"
    id = "xsh_bragg"
    description = "Calculation of crystal diffraction profile"
    icon = "icons/bragg.png"
    author = "create_widget.py"
    maintainer_email = "srio@esrf.eu"
    priority = 20
    category = ""
    keywords = ["oasys", "bragg"]

    outputs = [{"name":"BraggPreProcessorData",
                "type":BraggPreProcessorData,
                "doc":"Bragg PreProcessor Data",
                "id":"BraggPreProcessorData"}]

    want_main_area = True

    DESCRIPTOR = Setting(0)
    H_MILLER_INDEX = Setting(1)
    K_MILLER_INDEX = Setting(1)
    L_MILLER_INDEX = Setting(1)
    TEMPERATURE_FACTOR = Setting(1.0)
    E_MIN = Setting(5000.0)
    E_MAX = Setting(15000.0)
    E_STEP = Setting(100.0)
    SHADOW_FILE = Setting("bragg.dat")

    PREPROCESSOR_FILE_VERSION = Setting(1)
    DESCRIPTOR_DABAX = Setting(0)
    DESCRIPTOR_XRAYSERVER = Setting(129)

    IMAGE_WIDTH  = 860
    IMAGE_HEIGHT = 545

    MAX_WIDTH          = 1320
    MAX_HEIGHT         = 700
    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT   = 630

    #
    # Plots
    #
    plot_flag = Setting(0)
    scan_e_n = Setting(100)
    scan_e_delta = Setting(10.0)

    scan_a_n = Setting(100)
    scan_a_delta = Setting(100)
    scan_e0 = Setting(10000.0)

    usage_path = os.path.join(resources.package_dirname("orangecontrib.shadow4.widgets.gui"), "misc", "bragg_usage.png")

    bragg_dict = None

    #crystalpy (plots)
    calculation_method = 1         # 0=Zachariasen, 1=Guigay
    calculation_strategy_flag = 2  # 0=mpmath 1=numpy 2=numpy-truncated

    def __init__(self):
        super().__init__()

        self.populate_crystal_lists()

        self.runaction = widget.OWAction("Compute", self)
        self.runaction.triggered.connect(self.compute)
        self.addAction(self.runaction)

        self.setFixedWidth(self.MAX_WIDTH)
        self.setFixedHeight(self.MAX_HEIGHT)

        gui.separator(self.controlArea)

        box0 = oasysgui.widgetBox(self.controlArea, "",orientation="horizontal")
        button = gui.button(box0, self, "Compute", callback=self.compute)
        button.setFixedHeight(35)

        gui.separator(self.controlArea)

        tabs_setting = oasysgui.tabWidget(self.controlArea)
        tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT)
        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)

        tab_bas = oasysgui.createTabPage(tabs_setting, "Crystal Settings")
        self.populate_crystal_settings(tab_bas)

        tab_input_plot = oasysgui.createTabPage(tabs_setting, "Plots")
        self.populate_tab_plots(tab_input_plot)

        tab_usa = oasysgui.createTabPage(tabs_setting, "Use of the Widget")
        self.populate_tab_use_of_widget(tab_usa)

        self.main_tabs = oasysgui.tabWidget(self.mainArea)
        tab_out = oasysgui.createTabPage(self.main_tabs, "Output")
        self.plot_tab = oasysgui.createTabPage(self.main_tabs, "Plots (optional scan)")

        self.shadow_output = oasysgui.textArea()

        out_box = oasysgui.widgetBox(tab_out, "System Output", addSpace=True, orientation="horizontal", height=400)
        out_box.layout().addWidget(self.shadow_output)

        self.process_showers()

        gui.rubber(self.controlArea)

    def populate_crystal_lists(self):
        dx1 = DabaxDecorated(file_Crystals="Crystals.dat")
        try: list1 = dx1.Crystal_GetCrystalsList()
        except HTTPError as e: # Anti-bot policies can block this call
            if "UserAgentBlocked" in str(e): list1 = {}
            else: raise e
        self.crystals_dabax = list1

        dx2 = DabaxDecorated(file_Crystals="Crystals_xrayserver.dat")
        try: list2 = dx2.Crystal_GetCrystalsList()
        except HTTPError as e: # Anti-bot policies can block this call
            if "UserAgentBlocked" in str(e): list2 = {}
            else: raise e
        self.crystals_xrayserver = list2

    def populate_tab_use_of_widget(self, tab_usa):
        tab_usa.setStyleSheet("background-color: white;")
        usage_box = oasysgui.widgetBox(tab_usa, "", addSpace=True, orientation="horizontal")
        label = QLabel("")
        label.setAlignment(Qt.AlignCenter)
        label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        label.setPixmap(QPixmap(self.usage_path))
        usage_box.layout().addWidget(label)

    def populate_tab_plots(self, tab_plots):
        box = gui.widgetBox(tab_plots, "Optional scan plots", orientation="vertical")

        gui.comboBox(box, self, "plot_flag", tooltip="plot_flag",
                     label="Scan plot", addSpace=True,
                     items=['No', 'energy-scan', 'grazing angle-scan'],
                     valueType=int, orientation="horizontal", labelWidth=270, callback=self.do_plots)

        self.box_plots = gui.widgetBox(tab_plots, "Optional scan plots", orientation="vertical")

        self.box_plot_e = oasysgui.widgetBox(self.box_plots, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(self.box_plot_e, self, "scan_e_delta", label="delta [eV]", tooltip="scan_e_delta",
                          valueType=float, addSpace=True, labelWidth=100, orientation="horizontal",
                          callback=self.do_plots)
        oasysgui.lineEdit(self.box_plot_e, self, "scan_e_n", label="points", tooltip="scan_e_n",
                          valueType=int, addSpace=True, labelWidth=50, orientation="horizontal",
                          callback=self.do_plots)

        self.box_plot_a = oasysgui.widgetBox(self.box_plots, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(self.box_plot_a, self, "scan_a_delta", label="delta [urad]", tooltip="scan_a_delta",
                          valueType=float, addSpace=True, labelWidth=100, orientation="horizontal",
                          callback=self.do_plots)
        oasysgui.lineEdit(self.box_plot_a, self, "scan_a_n", label="points", tooltip="scan_a_n",
                          valueType=int, addSpace=True, labelWidth=50, orientation="horizontal",
                          callback=self.do_plots)

        self.box_plot_e0 = oasysgui.widgetBox(self.box_plots, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(self.box_plot_e0, self, "scan_e0", label="Fixed energy [eV]", tooltip="scan_e0",
                          valueType=float, addSpace=True, labelWidth=200, orientation="horizontal",
                          callback=self.do_plots)

    def populate_crystal_settings(self, tab_bas):
        #
        # basic settings
        #
        idx = -1

        box = oasysgui.widgetBox(tab_bas, "Crystal Parameters", orientation="vertical")

        # widget index -0.1
        idx += 1
        gui.comboBox(box, self, "PREPROCESSOR_FILE_VERSION", tooltip="PREPROCESSOR_FILE_VERSION",
                     label=self.unitLabels()[idx], addSpace=True,
                     # items=["v1 [default]","v2 [from DABAX list]","v2 [from XRayServer list]"],
                     items=["from DABAX list (version 2)", "from XRayServer list (version 2)"],
                     sendSelectedValue=False,
                     valueType=int, orientation="horizontal", labelWidth=350)
        self.show_at(self.unitFlags()[idx], box)

        # widget index 0.1
        idx += 1
        box2 = oasysgui.widgetBox(box, "", orientation="vertical")
        gui.comboBox(box2, self, "DESCRIPTOR_DABAX", tooltip="DESCRIPTOR_DABAX",
                     label=self.unitLabels()[idx], addSpace=True,
                     items=self.crystals_dabax, sendSelectedValue=False,
                     valueType=int, orientation="horizontal", labelWidth=350)
        self.show_at(self.unitFlags()[idx], box2)

        # widget index 0.2
        idx += 1
        box3 = oasysgui.widgetBox(box, "", orientation="vertical")
        gui.comboBox(box3, self, "DESCRIPTOR_XRAYSERVER", tooltip="DESCRIPTOR_XRAYSERVER",
                     label=self.unitLabels()[idx], addSpace=True,
                     items=self.crystals_xrayserver, sendSelectedValue=False,
                     valueType=int, orientation="horizontal", labelWidth=350)
        self.show_at(self.unitFlags()[idx], box3)

        # widget index 1
        idx += 1
        box_miller = oasysgui.widgetBox(box, "", orientation="horizontal")
        oasysgui.lineEdit(box_miller, self, "H_MILLER_INDEX", tooltip="H_MILLER_INDEX",
                          label="Miller Indices [h k l]", addSpace=True,
                          valueType=int, labelWidth=350, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box_miller)

        # widget index 2
        idx += 1
        oasysgui.lineEdit(box_miller, self, "K_MILLER_INDEX", tooltip="K_MILLER_INDEX", addSpace=True,
                          valueType=int)
        self.show_at(self.unitFlags()[idx], box)

        # widget index 3
        idx += 1
        oasysgui.lineEdit(box_miller, self, "L_MILLER_INDEX", tooltip="L_MILLER_INDEX",
                          addSpace=True,
                          valueType=int, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

        gui.separator(box)

        # widget index 4
        idx += 1
        oasysgui.lineEdit(box, self, "TEMPERATURE_FACTOR", tooltip="TEMPERATURE_FACTOR",
                          label=self.unitLabels()[idx], addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

        # widget index 5
        idx += 1
        oasysgui.lineEdit(box, self, "E_MIN", tooltip="E_MIN",
                          label=self.unitLabels()[idx], addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

        # widget index 6
        idx += 1
        oasysgui.lineEdit(box, self, "E_MAX", tooltip="E_MAX",
                          label=self.unitLabels()[idx], addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

        # widget index 7
        idx += 1
        oasysgui.lineEdit(box, self, "E_STEP", tooltip="E_STEP",
                          label=self.unitLabels()[idx], addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

        # widget index 8
        idx += 1
        box_2 = oasysgui.widgetBox(box, "", addSpace=True, orientation="horizontal")

        self.le_SHADOW_FILE = oasysgui.lineEdit(box_2, self, "SHADOW_FILE", tooltip="SHADOW_FILE",
                                                label=self.unitLabels()[idx], addSpace=True, labelWidth=180,
                                                orientation="horizontal")

        gui.button(box_2, self, "...", callback=self.selectFile)

        self.show_at(self.unitFlags()[idx], box)

    def unitLabels(self):
         return ['Preprocessor file version','Crystal descriptor [DABAX list]','Crystal descriptor [XRayServer list]','H miller index','K miller index','L miller index','Temperature factor','Minimum energy [eV]','Maximum energy [eV]','Energy step [eV]','File name (for SHADOW)']

    def unitFlags(self):
         return ['True','self.PREPROCESSOR_FILE_VERSION == 0','self.PREPROCESSOR_FILE_VERSION == 1','True','True','True','True','True','True','True','True']

    def selectFile(self):
        self.le_SHADOW_FILE.setText(oasysgui.selectFileFromDialog(self, self.SHADOW_FILE, "Select Output File"))

    def set_visibility(self):
        self.box_plots.setVisible(self.plot_flag > 0)
        if self.plot_flag == 1:
            self.box_plot_e.setVisible(True)
            self.box_plot_a.setVisible(False)
            self.box_plot_e0.setVisible(True)
        elif self.plot_flag == 2:
            self.box_plot_e.setVisible(False)
            self.box_plot_a.setVisible(True)
            self.box_plot_e0.setVisible(True)

    def compute(self):
        sys.stdout = EmittingStream(textWritten=self.writeStdOut)
        self.checkFields()

        if self.PREPROCESSOR_FILE_VERSION == 0:
            descriptor=self.crystals_dabax[self.DESCRIPTOR_DABAX]
            material_constants_library = DabaxDecorated(file_Crystals="Crystals.dat")
        else:
            descriptor = self.crystals_xrayserver[self.DESCRIPTOR_XRAYSERVER]
            material_constants_library = DabaxDecorated(file_Crystals="Crystals_xrayserver.dat")

        self.bragg_dict = create_bragg_preprocessor_file_v2(interactive=False,
                                          DESCRIPTOR=descriptor,
                                          H_MILLER_INDEX=self.H_MILLER_INDEX,
                                          K_MILLER_INDEX=self.K_MILLER_INDEX,
                                          L_MILLER_INDEX=self.L_MILLER_INDEX,
                                          TEMPERATURE_FACTOR=self.TEMPERATURE_FACTOR,
                                          E_MIN=self.E_MIN,
                                          E_MAX=self.E_MAX,
                                          E_STEP=self.E_STEP,
                                          SHADOW_FILE=congruence.checkFileName(self.SHADOW_FILE),
                                          material_constants_library=material_constants_library,
                                          )

        self.send("BraggPreProcessorData", BraggPreProcessorData(bragg_data_file=self.SHADOW_FILE))
        self.do_plots()

    def checkFields(self):
        self.H_MILLER_INDEX = congruence.checkNumber(self.H_MILLER_INDEX, "H miller index")
        self.K_MILLER_INDEX = congruence.checkNumber(self.K_MILLER_INDEX, "K miller index")
        self.L_MILLER_INDEX = congruence.checkNumber(self.L_MILLER_INDEX, "L miller index")
        self.TEMPERATURE_FACTOR = congruence.checkNumber(self.TEMPERATURE_FACTOR, "Temperature factor")
        self.E_MIN  = congruence.checkPositiveNumber(self.E_MIN , "Minimum energy")
        self.E_MAX  = congruence.checkStrictlyPositiveNumber(self.E_MAX , "Maximum Energy")
        self.E_STEP = congruence.checkStrictlyPositiveNumber(self.E_STEP, "Energy step")
        congruence.checkLessOrEqualThan(self.E_MIN, self.E_MAX, "From Energy", "To Energy")
        congruence.checkDir(self.SHADOW_FILE)

    def do_plots(self):
        self.set_visibility()
        if self.bragg_dict is None: self.compute()
        self.plot_tab.layout().removeItem(self.plot_tab.layout().itemAt(0))

        if self.plot_flag == 0:
            plot_widget_id = plot_data1D([0], [0], xtitle="", ytitle="")
        elif self.plot_flag == 1:
            energy_array, R_S_array, R_P_array = self.calculate_simple_diffraction_energy_scan()
            plot_widget_id = plot_multi_data1D(energy_array, [R_S_array, R_P_array], xtitle="Photon energy [eV]",
                                         ytitle="Reflectivity", ytitles=['S-polaized','P-polarized'])
        elif self.plot_flag == 2:
            theta_array, R_S_array, R_P_array = self.calculate_simple_diffraction_theta_scan()
            plot_widget_id = plot_multi_data1D(1e6 * numpy.array(theta_array),
                                        [numpy.array(R_S_array, dtype=float), numpy.array(R_P_array, dtype=float)],
                                        xtitle="theta - theta_B [urad]", ytitle="Reflectivity-S",
                                        ytitles=['S-polaized','P-polarized'])
        self.plot_tab.layout().addWidget(plot_widget_id)

    def calculate_simple_diffraction_theta_scan(self):
        print("\nCreating a diffraction setup (shadow preprocessor file V2)...")
        diffraction_setup = DiffractionSetupShadowPreprocessorV2(geometry_type=BraggDiffraction(),  # todo: use oe._diffraction_geometry
                                             crystal_name="",  # string
                                             thickness=1e-2,  # meters
                                             miller_h=self.H_MILLER_INDEX,          # int
                                             miller_k=self.K_MILLER_INDEX,          # int
                                             miller_l=self.L_MILLER_INDEX,          # int
                                             asymmetry_angle=0.0,  # radians
                                             azimuthal_angle=0.0,
                                             preprocessor_file=self.SHADOW_FILE)

        angle_deviation_min = -0.5 * self.scan_a_delta * 1e-6  # radians
        angle_deviation_max = 0.5 * self.scan_a_delta * 1e-6 # radians
        angle_deviation_points = self.scan_a_n
        angle_step = (angle_deviation_max - angle_deviation_min) / angle_deviation_points
        #
        # gets Bragg angle needed to create deviation's scan
        #
        bragg_angle = diffraction_setup.angleBragg(self.scan_e0)
        print("Bragg angle for E=%f eV is %f deg" % (self.scan_e0, bragg_angle * 180.0 / numpy.pi))
        deviations = numpy.zeros(angle_deviation_points)
        bunch_in = ComplexAmplitudePhotonBunch()
        K0 = diffraction_setup.vectorK0(self.scan_e0)
        K0unitary = K0.getNormalizedVector()

        for ia in range(angle_deviation_points):
            deviation = angle_deviation_min + ia * angle_step
            # minus sign in angle is to perform cw rotation when deviation increses
            Vin = K0unitary.rotateAroundAxis(Vector(1, 0, 0), -deviation)
            photon = ComplexAmplitudePhoton(energy_in_ev=self.scan_e0, direction_vector=Vin)

            bunch_in.addPhoton(photon)
            deviations[ia] = angle_deviation_min + ia * angle_step

        coeffs = Diffraction.calculateDiffractedComplexAmplitudes(diffraction_setup,
                                                                  bunch_in,
                                                                  is_thick=1,
                                                                  calculation_method=self.calculation_method,
                                                                  calculation_strategy_flag=self.calculation_strategy_flag)
        intensityS = numpy.abs(coeffs["S"]) ** 2
        intensityP = numpy.abs(coeffs["P"]) ** 2

        return deviations, intensityS, intensityP

    def calculate_simple_diffraction_energy_scan(self):
        print("\nCreating a diffraction setup (shadow preprocessor file V2)...")
        diffraction_setup = DiffractionSetupShadowPreprocessorV2(geometry_type=BraggDiffraction(),  # todo: use oe._diffraction_geometry
                                             crystal_name="",  # string
                                             thickness=1e-2,  # meters
                                             miller_h=self.H_MILLER_INDEX,          # int
                                             miller_k=self.K_MILLER_INDEX,          # int
                                             miller_l=self.L_MILLER_INDEX,          # int
                                             asymmetry_angle=0.0,  # radians
                                             azimuthal_angle=0.0,
                                             preprocessor_file=self.SHADOW_FILE)

        diffraction = Diffraction()
        energies = numpy.linspace(self.scan_e0 - 0.5 * self.scan_e_delta,
                                  self.scan_e0 + 0.5 * self.scan_e_delta,
                                  self.scan_e_n)
        scan = numpy.zeros_like(energies)
        intensityS = numpy.zeros_like(scan, dtype=float)
        intensityP = numpy.zeros_like(scan, dtype=float)
        r = numpy.zeros_like(energies)
        bragg_angle = diffraction_setup.angleBragg(self.scan_e0)
        print("Bragg angle for E=%f eV is %f deg" % (self.scan_e0, bragg_angle * 180.0 / numpy.pi))

        for i in range(energies.size):
            #
            # gets Bragg angle needed to create deviation's scan
            #
            energy = energies[i]
            # Create a Diffraction object (the calculator)
            deviation = 0.0 # angle_deviation_min + ia * angle_step
            angle = deviation  + bragg_angle
            # calculate the components of the unitary vector of the incident photon scan
            # Note that diffraction plane is YZ
            yy = numpy.cos(angle)
            zz = - numpy.abs(numpy.sin(angle))
            photon = Photon(energy_in_ev=energy,direction_vector=Vector(0.0,yy,zz))
            # perform the calculation
            coeffs_r = Diffraction.calculateDiffractedComplexAmplitudes(diffraction_setup,
                                                                        photon,
                                                                        is_thick=1,
                                                                        calculation_method=self.calculation_method,
                                                                        calculation_strategy_flag=self.calculation_strategy_flag)
            scan[i] = energy
            intensityS[i] = numpy.abs(coeffs_r["S"]) ** 2
            intensityP[i] = numpy.abs(coeffs_r["P"]) ** 2
        return scan, intensityS, intensityP


    def writeStdOut(self, text):
        cursor = self.shadow_output.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.insertText(text)
        self.shadow_output.setTextCursor(cursor)
        self.shadow_output.ensureCursorVisible()

if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = OWBragg()
    w.PREPROCESSOR_FILE_VERSION = 1
    w.show()
    app.exec()
    w.saveSettings()



from syned.beamline.shape import Circle
from shadow4.beamline.optical_elements.refractors.s4_lens import S4Lens, S4LensElement
from orangecontrib.shadow4.widgets.gui.ow_abstract_lens import OWAbstractLens

class OWLens(OWAbstractLens):
    name = "Refractive Lens"
    description = "Shadow Refractive Lens"
    icon = "icons/lens.png"
    priority = 2.1

    def __init__(self):
        super().__init__()

    # ----------------------------------------------------
    # from OpticalElement

    def get_optical_element_instance(self):
        try:    name = self.getNode().title
        except: name = "Refractive Lens"

        um_to_si = 1e-6

        if self.has_finite_diameter == 0: boundary_shape = Circle(radius=um_to_si*self.diameter*0.5)
        else:                             boundary_shape = None
        if self.is_cylinder == 1: cylinder_angle = self.cylinder_angle + 1
        else:                     cylinder_angle = 0

        return S4Lens(name=name,
                      boundary_shape=boundary_shape,
                      material=self.material,
                      density=self.density,
                      thickness=self.interthickness*um_to_si,
                      surface_shape=self.surface_shape,
                      convex_to_the_beam=self.convex_to_the_beam,
                      cylinder_angle=cylinder_angle,
                      ri_calculation_mode=self.ri_calculation_mode,
                      prerefl_file=self.prerefl_file,
                      refraction_index=self.refraction_index,
                      attenuation_coefficient=self.attenuation_coefficient,
                      radius=self.radius*um_to_si,
                      conic_coefficients1=None, # TODO: add conic coefficient shape to the GUI
                      conic_coefficients2=None,  # TODO: add conic coefficient shape to the GUI
                      )

    def get_beamline_element_instance(self):
        return S4LensElement()

if __name__ == "__main__":
    from shadow4.beamline.s4_beamline import S4Beamline
    import sys
    from orangecontrib.shadow4.util.shadow4_objects import ShadowData, PreReflPreProcessorData, VlsPgmPreProcessorData

    def get_test_beam():
        # electron beam
        from syned.storage_ring.light_source import ElectronBeam
        electron_beam = ElectronBeam(energy_in_GeV=6, energy_spread=0.001, current=0.2)
        electron_beam.set_sigmas_all(sigma_x=3.01836e-05, sigma_y=3.63641e-06, sigma_xp=4.36821e-06,
                                     sigma_yp=1.37498e-06)

        # Gaussian undulator
        from shadow4.sources.undulator.s4_undulator_gaussian import S4UndulatorGaussian
        sourceundulator = S4UndulatorGaussian(
            period_length=0.0159999,
            number_of_periods=100,
            photon_energy=2700.136,
            delta_e=0.0,
            flag_emittance=1,  # Use emittance (0=No, 1=Yes)
        )
        sourceundulator.set_energy_monochromatic(2700.14)

        from shadow4.sources.undulator.s4_undulator_gaussian_light_source import S4UndulatorGaussianLightSource
        light_source = S4UndulatorGaussianLightSource(name='GaussianUndulator', electron_beam=electron_beam,
                                              magnetic_structure=sourceundulator, nrays=5000, seed=5676561)

        beam = light_source.get_beam()

        return ShadowData(beam=beam, beamline=S4Beamline(light_source=light_source))

    from PyQt5.QtWidgets import QApplication
    a = QApplication(sys.argv)
    ow = OWLens()
    ow.set_shadow_data(get_test_beam())

    ow.show()
    a.exec_()
    ow.saveSettings()




import sys

from PyQt5 import QtGui
from PyQt5.QtCore import QRect
from PyQt5.QtWidgets import QApplication
from orangewidget import gui
from oasys.widgets import gui as oasysgui, widget
from oasys.widgets.gui import MessageDialog

from orangecontrib.shadow4.util.shadow4_objects import ShadowData
from orangecontrib.shadow4.util.shadow4_util import ShadowCongruence
from orangecontrib.shadow4.util.python_script import PythonScript, PythonConsole

from shadow4.tools.beamline_tools import flux_summary

class OWInfo(widget.OWWidget):

    name = "Info"
    description = "Display Data: Info"
    icon = "icons/info.png"
    maintainer = "Luca Rebuffi"
    maintainer_email = "srio(@at@)esrf.eu, lrebuffi(@at@)anl.gov"
    priority = 4
    category = "Data Display Tools"
    keywords = ["data", "file", "load", "read"]

    inputs = [("Shadow Data", ShadowData, "set_shadow_data")]

    WIDGET_WIDTH = 950
    WIDGET_HEIGHT = 650

    want_main_area=1
    want_control_area = 0

    input_data = None
    is_automatic_run = 1

    def __init__(self):
        super().__init__()

        geom = QApplication.desktop().availableGeometry()

        window_width  = round(min(geom.width()*0.98, self.WIDGET_WIDTH))
        window_height = round(min(geom.height() * 0.95, self.WIDGET_HEIGHT))

        self.setGeometry(QRect(round(geom.width()*0.05),
                               round(geom.height()*0.05),
                               window_width,
                               window_height))

        gen_box = gui.widgetBox(self.mainArea, "Beamline Info", addSpace=True, orientation="horizontal")

        tabs_setting1 = oasysgui.tabWidget(gen_box)
        tabs_setting1.setFixedHeight(self.WIDGET_HEIGHT-60)
        tabs_setting1.setFixedWidth(self.WIDGET_WIDTH-60)

        tab_flux = oasysgui.createTabPage(tabs_setting1, "Flux-Power")
        tab_sys = oasysgui.createTabPage(tabs_setting1, "Sys Info")
        tab_sys_plot_side = oasysgui.createTabPage(tabs_setting1, "Sys Plot (Side View)")
        tab_sys_plot_top = oasysgui.createTabPage(tabs_setting1, "Sys Plot (Bottom View)")
        tab_sys_plot_front = oasysgui.createTabPage(tabs_setting1, "Sys Plot (Front View)")
        tab_mir = oasysgui.createTabPage(tabs_setting1, "OE Info")
        tab_sou = oasysgui.createTabPage(tabs_setting1, "Source Info")
        tab_dis = oasysgui.createTabPage(tabs_setting1, "Distances Summary")
        tab_syned = oasysgui.createTabPage(tabs_setting1, "BL (syned info)")
        tab_syned_json = oasysgui.createTabPage(tabs_setting1, "BL (json)")
        # tab_scr = oasysgui.createTabPage(tabs_setting1, "Python Script")
        tab_out = oasysgui.createTabPage(tabs_setting1, "System Output")

        # flux
        self.fluxPower = oasysgui.textArea()
        self.fluxPower.setMaximumHeight(self.WIDGET_HEIGHT-100)

        # sysinfo and plots
        self.sysInfo = oasysgui.textArea()
        self.sysInfo.setMaximumHeight(self.WIDGET_HEIGHT-100)

        self.sysPlotSide = oasysgui.plotWindow(tab_sys_plot_side)
        self.sysPlotSide.setMaximumHeight(self.WIDGET_HEIGHT-100)

        self.sysPlotTop = oasysgui.plotWindow(tab_sys_plot_top)
        self.sysPlotTop.setMaximumHeight(self.WIDGET_HEIGHT-100)

        self.sysPlotFront = oasysgui.plotWindow(tab_sys_plot_front)
        self.sysPlotFront.setMaximumHeight(self.WIDGET_HEIGHT-100)

        # mirinfo
        self.mirInfo = oasysgui.textArea()
        self.mirInfo.setMaximumHeight(self.WIDGET_HEIGHT-100)

        # sourcinfo
        self.sourceInfo = oasysgui.textArea()
        self.sourceInfo.setMaximumHeight(self.WIDGET_HEIGHT-100)

        # syned doc
        self.synedInfo = oasysgui.textArea()
        self.synedInfo.setMaximumHeight(self.WIDGET_HEIGHT-100)

        # syned json
        self.synedJson = oasysgui.textArea()
        self.synedJson.setMaximumHeight(self.WIDGET_HEIGHT-100)

        # distances
        self.distancesSummary = oasysgui.textArea()
        self.distancesSummary.setMaximumHeight(self.WIDGET_HEIGHT-100)

        # sysinfo plots
        flux_power_box = oasysgui.widgetBox(tab_flux, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        flux_power_box.layout().addWidget(self.fluxPower)

        sys_box = oasysgui.widgetBox(tab_sys, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        sys_box.layout().addWidget(self.sysInfo)

        sys_plot_side_box = oasysgui.widgetBox(tab_sys_plot_side, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        sys_plot_side_box.layout().addWidget(self.sysPlotSide)

        sys_plot_top_box = oasysgui.widgetBox(tab_sys_plot_top, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        sys_plot_top_box.layout().addWidget(self.sysPlotTop)

        sys_plot_front_box = oasysgui.widgetBox(tab_sys_plot_front, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        sys_plot_front_box.layout().addWidget(self.sysPlotFront)

        mir_box = oasysgui.widgetBox(tab_mir, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        mir_box.layout().addWidget(self.mirInfo)

        source_box = oasysgui.widgetBox(tab_sou, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        source_box.layout().addWidget(self.sourceInfo)

        syned_box = oasysgui.widgetBox(tab_syned, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        syned_box.layout().addWidget(self.synedInfo)

        syned_json_box = oasysgui.widgetBox(tab_syned_json, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        syned_json_box.layout().addWidget(self.synedJson)

        dist_box = oasysgui.widgetBox(tab_dis, "", addSpace=True, orientation="horizontal", height = self.WIDGET_HEIGHT-80, width = self.WIDGET_WIDTH-80)
        dist_box.layout().addWidget(self.distancesSummary)

        # # script
        # self.shadow4_script = PythonScript()
        # script_box = gui.widgetBox(tab_scr, "Python script", addSpace=True, orientation="horizontal")
        # script_box.layout().addWidget(self.shadow4_script)
        # # self.shadow4_script.code_area.setFixedHeight(400)
        #
        #
        # console
        # console_box = oasysgui.widgetBox(script_box, "", addSpace=True, orientation="vertical",
        #                                   height=150, width=self.WIDGET_WIDTH - 80)
        # self.console = PythonConsole(self.__dict__, self)
        # console_box.layout().addWidget(self.console)
        #
        self.shadow_output = oasysgui.textArea()

        out_box = oasysgui.widgetBox(tab_out, "System Output", addSpace=True, orientation="horizontal", height=self.WIDGET_HEIGHT - 80)
        out_box.layout().addWidget(self.shadow_output)

        self.refresh()

    def set_shadow_data(self, shadow_data : ShadowData):
        if ShadowCongruence.check_empty_data(shadow_data):
            if ShadowCongruence.check_empty_beam(shadow_data.beam):
                self.input_data = shadow_data
                self.refresh()
            else:
                MessageDialog.message(self, "Data not displayable: bad content", "Error", "critical")

    def refresh(self):

        self.fluxPower.setText("")
        self.sysInfo.setText("")
        self.mirInfo.setText("")
        self.sourceInfo.setText("")
        self.distancesSummary.setText("")
        self.synedJson.setText("")
        self.synedInfo.setText("")

        # self.shadow4_script.set_code("#")

        if self.input_data is None: return

        try:
            self.fluxPower.append(flux_summary(self.input_data.beamline))
        except:
            self.fluxPower.append("error in creating fluxPower")

        try:
            self.sysInfo.append(self.input_data.beamline.sysinfo())
        except:
            self.sysInfo.append("error in creating sysInfo")

        try:
            self.mirInfo.append(self.input_data.beamline.oeinfo())
        except:
            self.mirInfo.append("error in creating mirInfo")

        try:
            self.sourceInfo.append( self.input_data.beamline.sourcinfo())
        except:
            self.sourceInfo.append("error in creating xsourceInfoxx")

        try:
            self.synedInfo.append(self.input_data.beamline.info())
        except:
            self.synedInfo.append("error in creating synedInfo")

        try:
            self.synedJson.append(self.input_data.beamline.to_json())
        except:
            self.synedJson.append("error in creating json dump")

        try:
            self.distancesSummary.append(self.input_data.beamline.distances_summary())
        except:
            self.distancesSummary.append("error in creating xxdistancesSummaryx")

        try:
            self.sysplots()
        except:
            self.outputs.append("error in creating sysplots")

    def sysplots(self):
        try:
            status = 0
            dic = self.input_data.beamline.syspositions()

            status = 1
            self.sysPlotSide.addCurve(dic["optical_axis_y"], dic["optical_axis_z"], symbol='o', replace=True)
            self.sysPlotSide.setGraphXLabel("Y [m]")
            self.sysPlotSide.setGraphYLabel("Z [m]")
            self.sysPlotSide.setGraphTitle("Side View of optical axis")
            self.sysPlotSide.replot()

            status = 2
            self.sysPlotTop.addCurve(dic["optical_axis_y"], dic["optical_axis_x"], symbol='o', replace=True)
            self.sysPlotTop.setGraphXLabel("Y [m]")
            self.sysPlotTop.setGraphYLabel("X [m]")
            self.sysPlotTop.setGraphTitle("Bottom View of optical axis")
            self.sysPlotTop.replot()

            status = 3
            self.sysPlotFront.addCurve(dic["optical_axis_x"], dic["optical_axis_z"], symbol='o', replace=True)
            self.sysPlotFront.setGraphXLabel("X [m]")
            self.sysPlotFront.setGraphYLabel("Z [m]")
            self.sysPlotFront.setGraphTitle("Front View of optical axis")
            self.sysPlotFront.replot()

            status = 4
        except:
            self.shadow_output.setText(
                "Problem in plotting SysPlot. status: %d\n" % status)


    def writeStdOut(self, text):
        cursor = self.shadow_output.textCursor()
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertText(text)
        self.shadow_output.setTextCursor(cursor)
        self.shadow_output.ensureCursorVisible()


if __name__ == "__main__":
    from shadow4.beamline.s4_beamline import S4Beamline

    beamline = S4Beamline()

    # electron beam
    from shadow4.sources.s4_electron_beam import S4ElectronBeam

    electron_beam = S4ElectronBeam(energy_in_GeV=6, energy_spread=0.001, current=0.2)
    electron_beam.set_sigmas_all(sigma_x=3.01836e-05, sigma_y=4.36821e-06, sigma_xp=3.63641e-06, sigma_yp=1.37498e-06)

    # magnetic structure
    from shadow4.sources.undulator.s4_undulator_gaussian import S4UndulatorGaussian

    source = S4UndulatorGaussian(
        period_length=0.042,  # syned Undulator parameter (length in m)
        number_of_periods=38.571,  # syned Undulator parameter
        photon_energy=15000.0,  # Photon energy (in eV)
        delta_e=2.0,  # Photon energy width (in eV)
        ng_e=100,  # Photon energy scan number of points
        flag_emittance=1,  # when sampling rays: Use emittance (0=No, 1=Yes)
        flag_energy_spread=0,  # when sampling rays: Use e- energy spread (0=No, 1=Yes)
        harmonic_number=1,  # harmonic number
        flag_autoset_flux_central_cone=0,  # value to set the flux peak
        flux_central_cone=10000000000.0,  # value to set the flux peak
    )

    # light source
    from shadow4.sources.undulator.s4_undulator_gaussian_light_source import S4UndulatorGaussianLightSource

    light_source = S4UndulatorGaussianLightSource(name='GaussianUndulator', electron_beam=electron_beam,
                                                  magnetic_structure=source, nrays=15000, seed=5676561)
    beam = light_source.get_beam()

    beamline.set_light_source(light_source)

    # optical element number XX
    from syned.beamline.shape import Rectangle

    boundary_shape = Rectangle(x_left=-0.001, x_right=0.001, y_bottom=-0.001, y_top=0.001)

    from shadow4.beamline.optical_elements.absorbers.s4_screen import S4Screen

    optical_element = S4Screen(name='Generic Beam Screen/Slit/Stopper/Attenuator', boundary_shape=boundary_shape,
                               i_abs=0,  # 0=No, 1=prerefl file_abs, 2=xraylib, 3=dabax
                               i_stop=0, thick=0, file_abs='<specify file name>', material='Au', density=19.3)

    from syned.beamline.element_coordinates import ElementCoordinates

    coordinates = ElementCoordinates(p=27.2, q=0, angle_radial=0, angle_azimuthal=0, angle_radial_out=3.14159)
    from shadow4.beamline.optical_elements.absorbers.s4_screen import S4ScreenElement

    beamline_element = S4ScreenElement(optical_element=optical_element, coordinates=coordinates, input_beam=beam)

    beam, footprint = beamline_element.trace_beam()

    beamline.append_beamline_element(beamline_element)

    # optical element number XX
    boundary_shape = None

    from shadow4.beamline.optical_elements.mirrors.s4_plane_mirror import S4PlaneMirror

    optical_element = S4PlaneMirror(name='Plane Mirror', boundary_shape=boundary_shape,
                                    f_reflec=1, f_refl=5, file_refl='<none>', refraction_index=0.99999 + 0.001j,
                                    coating_material='Ni', coating_density=8.902, coating_roughness=0)

    from syned.beamline.element_coordinates import ElementCoordinates

    coordinates = ElementCoordinates(p=2.7, q=0, angle_radial=1.5638, angle_azimuthal=1.5708, angle_radial_out=1.5638)
    movements = None
    from shadow4.beamline.optical_elements.mirrors.s4_plane_mirror import S4PlaneMirrorElement

    beamline_element = S4PlaneMirrorElement(optical_element=optical_element, coordinates=coordinates,
                                            movements=movements, input_beam=beam)

    beam, mirr = beamline_element.trace_beam()

    beamline.append_beamline_element(beamline_element)

    # optical element number XX
    boundary_shape = None

    from shadow4.beamline.optical_elements.mirrors.s4_plane_mirror import S4PlaneMirror

    optical_element = S4PlaneMirror(name='Plane Mirror', boundary_shape=boundary_shape,
                                    f_reflec=1, f_refl=5, file_refl='<none>', refraction_index=0.99999 + 0.001j,
                                    coating_material='Ni', coating_density=8.902, coating_roughness=0)

    from syned.beamline.element_coordinates import ElementCoordinates

    coordinates = ElementCoordinates(p=0.825, q=0, angle_radial=1.5638, angle_azimuthal=3.14159,
                                     angle_radial_out=1.5638)
    movements = None
    from shadow4.beamline.optical_elements.mirrors.s4_plane_mirror import S4PlaneMirrorElement

    beamline_element = S4PlaneMirrorElement(optical_element=optical_element, coordinates=coordinates,
                                            movements=movements, input_beam=beam)

    beam, mirr = beamline_element.trace_beam()

    beamline.append_beamline_element(beamline_element)

    # optical element number XX

    from shadow4.beamline.optical_elements.ideal_elements.s4_empty import S4Empty

    optical_element = S4Empty(name='Empty Element')

    from syned.beamline.element_coordinates import ElementCoordinates

    coordinates = ElementCoordinates(p=0, q=0, angle_radial=0, angle_azimuthal=4.71239, angle_radial_out=3.14159)
    from shadow4.beamline.optical_elements.ideal_elements.s4_empty import S4EmptyElement

    beamline_element = S4EmptyElement(optical_element=optical_element, coordinates=coordinates, input_beam=beam)

    beam, mirr = beamline_element.trace_beam()

    beamline.append_beamline_element(beamline_element)

    # optical element number XX
    from syned.beamline.shape import Rectangle

    boundary_shape = Rectangle(x_left=-0.0015, x_right=0.0015, y_bottom=-0.0015, y_top=0.0015)

    from shadow4.beamline.optical_elements.absorbers.s4_screen import S4Screen

    optical_element = S4Screen(name='Generic Beam Screen/Slit/Stopper/Attenuator', boundary_shape=boundary_shape,
                               i_abs=0,  # 0=No, 1=prerefl file_abs, 2=xraylib, 3=dabax
                               i_stop=0, thick=0, file_abs='<specify file name>', material='Au', density=19.3)

    from syned.beamline.element_coordinates import ElementCoordinates

    coordinates = ElementCoordinates(p=5.475, q=0, angle_radial=0, angle_azimuthal=0, angle_radial_out=3.14159)
    from shadow4.beamline.optical_elements.absorbers.s4_screen import S4ScreenElement

    beamline_element = S4ScreenElement(optical_element=optical_element, coordinates=coordinates, input_beam=beam)

    beam, footprint = beamline_element.trace_beam()

    beamline.append_beamline_element(beamline_element)


###################################



    # test plot
    if False:
        from srxraylib.plot.gol import plot_scatter

        # plot_scatter(beam.get_photon_energy_eV(nolost=1), beam.get_column(23, nolost=1),
        #              title='(Intensity,Photon Energy)', plot_histograms=0)
        plot_scatter(1e6 * beam.get_column(1, nolost=1), 1e6 * beam.get_column(3, nolost=1), title='(X,Z) in microns')

    from PyQt5.QtWidgets import QApplication
    a = QApplication(sys.argv)
    ow = OWInfo()
    ow.set_shadow_data(ShadowData(beam=beam, footprint=footprint, number_of_rays=0, beamline=beamline))
    ow.show()
    a.exec_()



import numpy, copy

from PyQt5.QtWidgets import QWidget, QMessageBox, QVBoxLayout
from PyQt5.QtCore import Qt

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence
from oasys.widgets.gui import ConfirmDialog
from orangecontrib.shadow4.util.shadow4_util import ShadowPhysics

from syned.beamline.shape import Circle, Rectangle
from syned.beamline.element_coordinates import ElementCoordinates

from shadow4.beamline.optical_elements.refractors.s4_transfocator import S4Transfocator, S4TransfocatorElement

from orangecontrib.shadow.util.shadow_objects import ShadowPreProcessorData # TODO: remove
from orangecontrib.shadow4.util.shadow4_objects import PreReflPreProcessorData
from orangecontrib.shadow4.widgets.gui.ow_optical_element import OWOpticalElement



class OWTransfocator(OWOpticalElement):
    name = "Transfocator"
    description = "Transfocator"
    icon = "icons/transfocator.png"
    priority = 2.3

    NONE_SPECIFIED = "NONE SPECIFIED"

    n_lenses = Setting([4, 2])
    slots_empty = Setting([0, 0])
    piling_thickness = Setting([2.5, 2.5])

    empty_space_after_last_interface = Setting([0.0, 0.0])
    surface_shape = Setting([1, 1])
    convex_to_the_beam = Setting([0, 0])

    has_finite_diameter = Setting([0, 0])
    diameter = Setting([0.632, 0.894])

    is_cylinder = Setting([0, 0])
    cylinder_angle = Setting([0.0, 0.0])

    ri_calculation_mode = Setting([0, 0])
    prerefl_file = Setting([NONE_SPECIFIED, NONE_SPECIFIED])
    refraction_index = Setting([1.0, 1.0])
    attenuation_coefficient = Setting([0.0, 0.0])
    material = Setting(["Be", "Al"])
    density =  Setting([1.848, 2.7])

    radius = Setting([0.1, 0.2])
    thickness = Setting([0.03, 0.03])


    input_data = None
    is_automatic_run = 1


    inputs = copy.deepcopy(OWOpticalElement.inputs)
    inputs.append(("PreRefl PreProcessor Data", PreReflPreProcessorData, "set_PreReflPreProcessorData"))

    def __init__(self):
        super().__init__(has_footprint=False)

    def populate_tab_position(self, tab_position):
        self.orientation_box = oasysgui.widgetBox(tab_position, "Optical Element Orientation", addSpace=True,
                                                  orientation="vertical")

        oasysgui.lineEdit(self.orientation_box, self, "source_plane_distance",
                          "Source Plane Distance to First Interface (P) [m]", labelWidth=260,
                          valueType=float, orientation="horizontal", tooltip="source_plane_distance")
        oasysgui.lineEdit(self.orientation_box, self, "image_plane_distance",
                          "Last Interface Distance to Image plane (Q) [m]", labelWidth=260,
                          valueType=float, orientation="horizontal", tooltip="image_plane_distance")

    def create_basic_settings_subtabs(self, tabs_basic_settings):
        return oasysgui.createTabPage(tabs_basic_settings, "Transfocator")  # to be populated

    def populate_basic_setting_subtabs(self, basic_setting_subtabs):

        tabs_button_box = oasysgui.widgetBox(basic_setting_subtabs, "", addSpace=False, orientation="horizontal")
        btns = [gui.button(tabs_button_box, self, "Insert CRL Before", callback=self.crl_insert_before),
                gui.button(tabs_button_box, self, "Insert CRL After", callback=self.crl_insert_after),
                gui.button(tabs_button_box, self, "Remove CRL", callback=self.crl_remove)]
        for btn in btns: btn.setFixedHeight(40)

        self.tab_crls = oasysgui.tabWidget(basic_setting_subtabs)
        self.crl_box_array = []

        for index in range(len(self.material)):
            tab_crl = oasysgui.createTabPage(self.tab_crls, "CRL " + str(index + 1))

            crl_box = CRLBox(transfocator=self,
                             parent=tab_crl,
                             n_lenses=self.n_lenses[index],
                             slots_empty=self.slots_empty[index],
                             piling_thickness=self.piling_thickness[index],
                             empty_space_after_last_interface=self.empty_space_after_last_interface[index],
                             surface_shape=self.surface_shape[index],
                             convex_to_the_beam=self.convex_to_the_beam[index],
                             has_finite_diameter=self.has_finite_diameter[index],
                             diameter=self.diameter[index],
                             is_cylinder=self.is_cylinder[index],
                             cylinder_angle=self.cylinder_angle[index],
                             ri_calculation_mode=self.ri_calculation_mode[index],
                             prerefl_file=self.prerefl_file[index],
                             refraction_index=self.refraction_index[index],
                             attenuation_coefficient=self.attenuation_coefficient[index],
                             material=self.material[index],
                             density=self.density[index],
                             radius=self.radius[index],
                             thickness=self.thickness[index],
                             )

            self.crl_box_array.append(crl_box)

    def get_optical_element_instance(self):
        try:
            name = self.getNode().title
        except:
            name = "Transfocator"

        um_to_si = 1e-6
        mm_to_si = 1e-3

        if self.has_finite_diameter[0] == 0:
            boundary_shape = None
        elif self.has_finite_diameter[0] == 1:
            boundary_shape = Circle(radius=um_to_si * self.diameter[0] * 0.5)
        elif self.has_finite_diameter[0] == 2:
            rr = um_to_si * self.diameter[0] * 0.5
            boundary_shape = Rectangle() (x_left=-rr, x_right=rr, y_bottom=-rr, y_top=rr)


        n = len(self.n_lenses)
        cylinder_angle = [0] * n
        piling_thickness = [0] * n
        thickness = [0] * n
        radius = [0] * n
        for i in range(n):
            if self.is_cylinder[i] == 1: cylinder_angle[i] = self.cylinder_angle[i] + 1
            thickness[i] = self.thickness[i] * um_to_si
            piling_thickness[i] = self.piling_thickness[i] * mm_to_si
            radius[i] = self.radius[i] * um_to_si

        optical_element = S4Transfocator(name='TF',
                                        n_lens=self.n_lenses,
                                        thickness=thickness,  # syned stuff
                                        boundary_shape=boundary_shape,
                                        material=self.material,
                                        density=self.density,
                                        piling_thickness=piling_thickness,
                                        surface_shape=self.surface_shape,
                                        convex_to_the_beam=self.convex_to_the_beam,
                                        cylinder_angle=cylinder_angle,
                                        ri_calculation_mode=self.ri_calculation_mode,
                                        prerefl_file=self.prerefl_file,
                                        refraction_index=self.refraction_index,
                                        attenuation_coefficient=self.attenuation_coefficient,
                                        dabax=None,
                                        radius=radius,
                                        conic_coefficients1=[[0] * 10] * n,
                                        conic_coefficients2=[[0] * 10] * n,
                                        empty_space_after_last_interface=[0.0] * n,
                                        )
        return optical_element

    def get_beamline_element_instance(self):

        beamline_element = S4TransfocatorElement(optical_element=self.get_optical_element_instance(),
                                                 coordinates=self.get_coordinates_instance(),
                                                 movements=self.get_movements_instance(),
                                                 input_beam=self.input_data.beam)
        return beamline_element

    def get_movements_instance(self): return None

    def get_coordinates_instance(self):
        return ElementCoordinates(
                p=self.source_plane_distance,
                q=self.image_plane_distance,
                angle_radial=0.0,
                angle_azimuthal=0.0,
                angle_radial_out=numpy.pi,
                )

    def crl_insert_before(self):
        current_index = self.tab_crls.currentIndex()

        if ConfirmDialog.confirmed(parent=self, message="Confirm Insertion of a new element before " + self.tab_crls.tabText(current_index) + "?"):
            tab_crl = oasysgui.widgetBox(self.tab_crls, addToLayout=0, margin=4)
            crl_box = CRLBox(transfocator=self, parent=tab_crl)

            self.tab_crls.insertTab(current_index, tab_crl, "TEMP")
            self.crl_box_array.insert(current_index, crl_box)
            self.dumpSettings()

            for index in range(current_index, self.tab_crls.count()):
                self.tab_crls.setTabText(index, "CRL " + str(index + 1))

            self.tab_crls.setCurrentIndex(current_index)

    def crl_insert_after(self):
        current_index = self.tab_crls.currentIndex()

        if ConfirmDialog.confirmed(parent=self, message="Confirm Insertion of a new element after " + self.tab_crls.tabText(current_index) + "?"):
            tab_crl = oasysgui.widgetBox(self.tab_crls, addToLayout=0, margin=4)
            crl_box = CRLBox(transfocator=self, parent=tab_crl)

            if current_index == self.tab_crls.count() - 1:  # LAST
                self.tab_crls.addTab(tab_crl, "TEMP")
                self.crl_box_array.append(crl_box)
            else:
                self.tab_crls.insertTab(current_index + 1, tab_crl, "TEMP")
                self.crl_box_array.insert(current_index + 1, crl_box)

            self.dumpSettings()

            for index in range(current_index, self.tab_crls.count()):
                self.tab_crls.setTabText(index, "CRL " + str(index + 1))

            self.tab_crls.setCurrentIndex(current_index + 1)

    def crl_remove(self):
        if self.tab_crls.count() <= 1:
            QMessageBox.critical(self, "Error",
                                       "Remove not possible, transfocator needs at least 1 element",
                                       QMessageBox.Ok)
        else:
            current_index = self.tab_crls.currentIndex()

            if ConfirmDialog.confirmed(parent=self, message="Confirm Removal of " + self.tab_crls.tabText(current_index) + "?"):
                self.tab_crls.removeTab(current_index)
                self.crl_box_array.pop(current_index)
                self.dumpSettings()

                for index in range(current_index, self.tab_crls.count()):
                    self.tab_crls.setTabText(index, "CRL " + str(index + 1))

                self.tab_crls.setCurrentIndex(current_index)

    def dumpSettings(self):
        bkp_n_lenses = copy.deepcopy(self.n_lenses)
        bkp_slots_empty = copy.deepcopy(self.slots_empty)
        bkp_piling_thickness = copy.deepcopy(self.piling_thickness)
        bkp_empty_space_after_last_interface = copy.deepcopy(self.empty_space_after_last_interface)
        bkp_surface_shape = copy.deepcopy(self.surface_shape)
        bkp_convex_to_the_beam = copy.deepcopy(self.convex_to_the_beam)
        bkp_has_finite_diameter = copy.deepcopy(self.has_finite_diameter)
        bkp_diameter = copy.deepcopy(self.diameter)
        bkp_is_cylinder = copy.deepcopy(self.is_cylinder)
        bkp_cylinder_angle = copy.deepcopy(self.cylinder_angle)
        bkp_ri_calculation_mode = copy.deepcopy(self.ri_calculation_mode)
        bkp_prerefl_file = copy.deepcopy(self.prerefl_file)
        bkp_refraction_index = copy.deepcopy(self.refraction_index)
        bkp_attenuation_coefficient = copy.deepcopy(self.attenuation_coefficient)
        bkp_material = copy.deepcopy(self.material)
        bkp_density = copy.deepcopy(self.density)
        bkp_radius = copy.deepcopy(self.radius)
        bkp_thickness = copy.deepcopy(self.thickness)

        try:
            self.n_lenses = []
            self.slots_empty = []
            self.piling_thickness = []
            self.empty_space_after_last_interface = []
            self.surface_shape = []
            self.convex_to_the_beam = []
            self.has_finite_diameter = []
            self.diameter = []
            self.is_cylinder = []
            self.cylinder_angle = []
            self.ri_calculation_mode = []
            self.prerefl_file = []
            self.refraction_index = []
            self.attenuation_coefficient = []
            self.material = []
            self.density = []
            self.radius = []
            self.thickness = []

            for index in range(len(self.crl_box_array)):
                self.n_lenses.append(self.crl_box_array[index].n_lenses)
                self.slots_empty.append(self.crl_box_array[index].slots_empty)
                self.piling_thickness.append(self.crl_box_array[index].piling_thickness)
                self.empty_space_after_last_interface.append(self.crl_box_array[index].empty_space_after_last_interface)
                self.surface_shape.append(self.crl_box_array[index].surface_shape)
                self.convex_to_the_beam.append(self.crl_box_array[index].convex_to_the_beam)
                self.has_finite_diameter.append(self.crl_box_array[index].has_finite_diameter)
                self.diameter.append(self.crl_box_array[index].diameter)
                self.is_cylinder.append(self.crl_box_array[index].is_cylinder)
                self.cylinder_angle.append(self.crl_box_array[index].cylinder_angle)
                self.ri_calculation_mode.append(self.crl_box_array[index].ri_calculation_mode)
                self.prerefl_file.append(self.crl_box_array[index].prerefl_file)
                self.refraction_index.append(self.crl_box_array[index].refraction_index)
                self.attenuation_coefficient.append(self.crl_box_array[index].attenuation_coefficient)
                self.material.append(self.crl_box_array[index].material)
                self.density.append(self.crl_box_array[index].density)
                self.radius.append(self.crl_box_array[index].radius)
                self.thickness.append(self.crl_box_array[index].thickness)
        except:
            self.n_lenses = copy.deepcopy(bkp_n_lenses)
            self.slots_empty = copy.deepcopy(bkp_slots_empty)
            self.piling_thickness = copy.deepcopy(bkp_piling_thickness)
            self.empty_space_after_last_interface = copy.deepcopy(bkp_empty_space_after_last_interface)
            self.surface_shape = copy.deepcopy(bkp_surface_shape)
            self.convex_to_the_beam = copy.deepcopy(bkp_convex_to_the_beam)
            self.has_finite_diameter = copy.deepcopy(bkp_has_finite_diameter)
            self.diameter = copy.deepcopy(bkp_diameter)
            self.is_cylinder = copy.deepcopy(bkp_is_cylinder)
            self.cylinder_angle = copy.deepcopy(bkp_cylinder_angle)
            self.ri_calculation_mode = copy.deepcopy(bkp_ri_calculation_mode)
            self.prerefl_file = copy.deepcopy(bkp_prerefl_file)
            self.refraction_index = copy.deepcopy(bkp_refraction_index)
            self.attenuation_coefficient = copy.deepcopy(bkp_attenuation_coefficient)
            self.material = copy.deepcopy(bkp_material)
            self.density = copy.deepcopy(bkp_density)
            self.radius = copy.deepcopy(bkp_radius)
            self.thickness = copy.deepcopy(bkp_thickness)

    ##############################
    # SINGLE FIELDS SIGNALS
    ##############################

    def dump_n_lenses(self):
        bkp_n_lenses = copy.deepcopy(self.n_lenses)

        try:
            self.n_lenses = []

            for index in range(len(self.crl_box_array)):
                self.n_lenses.append(self.crl_box_array[index].n_lenses)
        except:
            self.n_lenses = copy.deepcopy(bkp_n_lenses)

    def dump_slots_empty(self):
        bkp_slots_empty = copy.deepcopy(self.slots_empty)

        try:
            self.slots_empty = []

            for index in range(len(self.crl_box_array)):
                self.slots_empty.append(self.crl_box_array[index].slots_empty)
        except:
            self.slots_empty = copy.deepcopy(bkp_slots_empty)

    def dump_piling_thickness(self):
        bkp_piling_thickness = copy.deepcopy(self.piling_thickness)

        try:
            self.piling_thickness = []

            for index in range(len(self.crl_box_array)):
                self.piling_thickness.append(self.crl_box_array[index].piling_thickness)
        except:
            self.piling_thickness = copy.deepcopy(bkp_piling_thickness)

    def dump_empty_space_after_last_interface(self):
        bkp_empty_space_after_last_interface = copy.deepcopy(self.empty_space_after_last_interface)

        try:
            self.empty_space_after_last_interface = []

            for index in range(len(self.crl_box_array)):
                self.empty_space_after_last_interface.append(self.crl_box_array[index].empty_space_after_last_interface)
        except:
            self.empty_space_after_last_interface = copy.deepcopy(bkp_empty_space_after_last_interface)

    def dump_surface_shape(self):
        bkp_surface_shape = copy.deepcopy(self.surface_shape)

        try:
            self.surface_shape = []

            for index in range(len(self.crl_box_array)):
                self.surface_shape.append(self.crl_box_array[index].surface_shape)
        except:
            self.surface_shape = copy.deepcopy(bkp_surface_shape)

    def dump_convex_to_the_beam(self):
        bkp_convex_to_the_beam = copy.deepcopy(self.convex_to_the_beam)

        try:
            self.convex_to_the_beam = []

            for index in range(len(self.crl_box_array)):
                self.convex_to_the_beam.append(self.crl_box_array[index].convex_to_the_beam)
        except:
            self.convex_to_the_beam = copy.deepcopy(bkp_convex_to_the_beam)

    def dump_has_finite_diameter(self):
        bkp_has_finite_diameter = copy.deepcopy(self.has_finite_diameter)

        try:
            self.has_finite_diameter = []

            for index in range(len(self.crl_box_array)):
                self.has_finite_diameter.append(self.crl_box_array[index].has_finite_diameter)
        except:
            self.has_finite_diameter = copy.deepcopy(bkp_has_finite_diameter)

    def dump_diameter(self):
        bkp_diameter = copy.deepcopy(self.diameter)

        try:
            self.diameter = []

            for index in range(len(self.crl_box_array)):
                self.diameter.append(self.crl_box_array[index].diameter)
        except:
            self.diameter = copy.deepcopy(bkp_diameter)

    def dump_is_cylinder(self):
        bkp_is_cylinder = copy.deepcopy(self.is_cylinder)

        try:
            self.is_cylinder = []

            for index in range(len(self.crl_box_array)):
                self.is_cylinder.append(self.crl_box_array[index].is_cylinder)
        except:
            self.is_cylinder = copy.deepcopy(bkp_is_cylinder)

    def dump_cylinder_angle(self):
        bkp_cylinder_angle = copy.deepcopy(self.cylinder_angle)

        try:
            self.cylinder_angle = []

            for index in range(len(self.crl_box_array)):
                self.cylinder_angle.append(self.crl_box_array[index].cylinder_angle)
        except:
            self.cylinder_angle = copy.deepcopy(bkp_cylinder_angle)

    def dump_ri_calculation_mode(self):
        bkp_ri_calculation_mode = copy.deepcopy(self.ri_calculation_mode)

        try:
            self.ri_calculation_mode = []

            for index in range(len(self.crl_box_array)):
                self.ri_calculation_mode.append(self.crl_box_array[index].ri_calculation_mode)
        except:
            self.ri_calculation_mode = copy.deepcopy(bkp_ri_calculation_mode)

    def dump_prerefl_file(self):
        bkp_prerefl_file = copy.deepcopy(self.prerefl_file)

        try:
            self.prerefl_file = []

            for index in range(len(self.crl_box_array)):
                self.prerefl_file.append(self.crl_box_array[index].prerefl_file)
        except:
            self.prerefl_file = copy.deepcopy(bkp_prerefl_file)

    def dump_refraction_index(self):
        bkp_refraction_index = copy.deepcopy(self.refraction_index)

        try:
            self.refraction_index = []

            for index in range(len(self.crl_box_array)):
                self.refraction_index.append(self.crl_box_array[index].refraction_index)
        except:
            self.refraction_index = copy.deepcopy(bkp_refraction_index)

    def dump_attenuation_coefficient(self):
        bkp_attenuation_coefficient = copy.deepcopy(self.attenuation_coefficient)

        try:
            self.attenuation_coefficient = []

            for index in range(len(self.crl_box_array)):
                self.attenuation_coefficient.append(self.crl_box_array[index].attenuation_coefficient)
        except:
            self.attenuation_coefficient = copy.deepcopy(bkp_attenuation_coefficient)

    def dump_material(self):
        bkp_material = copy.deepcopy(self.material)

        try:
            self.material = []

            for index in range(len(self.crl_box_array)):
                self.material.append(self.crl_box_array[index].material)
        except:
            self.material = copy.deepcopy(bkp_material)

        self.set_Density()

    def set_Density(self):
        index = self.tab_crls.currentIndex()
        mm = self.material[index]
        if not mm is None:
            if not mm.strip() == "":
                mm = mm.strip()
                new_density = ShadowPhysics.getMaterialDensity(mm)
                self.crl_box_array[index].density = new_density
                self.dump_density()
                # Next line is added to display the value in the widget: I do not know why this is not automatic!
                self.crl_box_array[index].le_density.setText(str(new_density))

    def dump_density(self):
        bkp_density = copy.deepcopy(self.density)

        try:
            self.density = []

            for index in range(len(self.crl_box_array)):
                self.density.append(self.crl_box_array[index].density)
        except:
            self.density = copy.deepcopy(bkp_density)


    def dump_radius(self):
        bkp_radius = copy.deepcopy(self.radius)

        try:
            self.radius = []

            for index in range(len(self.crl_box_array)):
                self.radius.append(self.crl_box_array[index].radius)
        except:
            self.radius = copy.deepcopy(bkp_radius)

    def dump_thickness(self):
        bkp_thickness = copy.deepcopy(self.thickness)

        try:
            self.thickness = []

            for index in range(len(self.crl_box_array)):
                self.thickness.append(self.crl_box_array[index].thickness)
        except:
            self.thickness = copy.deepcopy(bkp_thickness)


    ############################################################
    #
    # USER INPUT MANAGEMENT
    #
    ############################################################

    def checkFields(self):
        for box in self.crl_box_array:
            box.checkFields()

    def setPreProcessorData(self, data): # TODO: remove
        if data is not None:
            if data.prerefl_data_file != ShadowPreProcessorData.NONE:
                for box in self.crl_box_array:
                    box.prerefl_file = data.prerefl_data_file
                    box.le_prerefl_file.setText(data.prerefl_data_file)
                    box.ri_calculation_mode = 1
                    box.ri_calculation_mode_combo.setCurrentIndex(1)

                    box.set_ri_calculation_mode()
            else:
                QMessageBox.warning(self, "Warning", "Incompatible Preprocessor Data", QMessageBox.Ok)

                self.dump_prerefl_file()

    def set_PreReflPreProcessorData(self, data):
        if data is not None:
            if data.prerefl_data_file != ShadowPreProcessorData.NONE:
                for box in self.crl_box_array:
                    box.prerefl_file = data.prerefl_data_file
                    box.le_prerefl_file.setText(data.prerefl_data_file)
                    box.ri_calculation_mode = 1
                    box.ri_calculation_mode_combo.setCurrentIndex(1)

                    box.set_ri_calculation_mode()
            else:
                QMessageBox.warning(self, "Warning", "Incompatible PreReflPreprocessor Data", QMessageBox.Ok)

                self.dump_prerefl_file()

    def setupUI(self):
        for box in self.crl_box_array:
            box.setupUI()


class CRLBox(QWidget):
    n_lenses = 30
    slots_empty = 0
    piling_thickness = 625e-4

    empty_space_after_last_interface = 0.0
    surface_shape = 1
    convex_to_the_beam = 1

    has_finite_diameter = 0
    diameter = 0.0

    is_cylinder = 1
    cylinder_angle = 0.0

    ri_calculation_mode = 0
    prerefl_file = OWTransfocator.NONE_SPECIFIED
    refraction_index = 1.0
    attenuation_coefficient = 0.0
    material = "Be"
    density = 1.848

    radius = 500e-2
    thickness = 0.001

    transfocator = None

    is_on_init = True

    def __init__(self,
                 transfocator=None,
                 parent=None,
                 n_lenses=30,
                 slots_empty=0,
                 piling_thickness=625e-4,
                 # p=0.0,
                 # q=0.0,
                 empty_space_after_last_interface=0.0,
                 surface_shape=1,
                 convex_to_the_beam=1,
                 has_finite_diameter=0,
                 diameter=0.0,
                 is_cylinder=1,
                 cylinder_angle=0.0,
                 ri_calculation_mode=0,
                 prerefl_file=OWTransfocator.NONE_SPECIFIED,
                 refraction_index=1.0,
                 attenuation_coefficient=0.0,
                 material="Be",
                 density=1.848,
                 radius=500e-2,
                 thickness=0.001,
                 ):
        super().__init__(parent)

        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignTop)
        self.setFixedWidth(470)
        self.setFixedHeight(700)

        self.transfocator = transfocator

        self.n_lenses = n_lenses
        self.slots_empty = slots_empty
        self.piling_thickness = piling_thickness
        self.empty_space_after_last_interface = empty_space_after_last_interface

        self.surface_shape = surface_shape
        self.convex_to_the_beam = convex_to_the_beam
        self.has_finite_diameter = has_finite_diameter
        self.diameter = diameter
        self.is_cylinder = is_cylinder
        self.cylinder_angle = cylinder_angle

        self.ri_calculation_mode = ri_calculation_mode
        self.prerefl_file = prerefl_file
        self.refraction_index = refraction_index
        self.attenuation_coefficient = attenuation_coefficient
        self.material = material
        self.density = density

        self.radius = radius
        self.thickness = thickness

        tabs0 = oasysgui.tabWidget(self, height=420, width=self.transfocator.CONTROL_AREA_WIDTH-35)

        tabs = oasysgui.widgetBox(tabs0, "", addSpace=False, orientation="vertical",
                                     width=self.transfocator.CONTROL_AREA_WIDTH - 45)

        crl_box = tabs

        oasysgui.lineEdit(crl_box, self, "n_lenses", "Number of lenses", tooltip="n_lenses[i]", labelWidth=260, valueType=int,
                          orientation="horizontal", callback=self.transfocator.dump_n_lenses)

        self.le_empty_space_after_last_interface = oasysgui.lineEdit(crl_box, self, "empty_space_after_last_interface",
                                    "Empty space after last CRL interface [m]", tooltip="empty_space_after_last_interface[i]",
                                    labelWidth=290, valueType=float, orientation="horizontal",
                                    callback=self.transfocator.dump_empty_space_after_last_interface)

        # optical constants
        ###############
        self.ri_calculation_mode_combo = gui.comboBox(crl_box, self, "ri_calculation_mode", tooltip="ri_calculation_mode[i]",
                                                      label="Refraction Index calculation mode", labelWidth=260,
                                                      items=["User Parameters", "Prerefl File", \
                                                             "Internal (using xraylib)", "Internal (using dabax)"],
                                                      sendSelectedValue=False, orientation="horizontal",
                                                      callback=self.set_ri_calculation_mode)

        self.calculation_mode_1 = oasysgui.widgetBox(crl_box, "", addSpace=False, orientation="vertical")
        oasysgui.lineEdit(self.calculation_mode_1, self, "refraction_index", "Refraction index", tooltip="refraction_index[i]",
                          labelWidth=260, valueType=float, orientation="horizontal",
                          callback=self.transfocator.dump_refraction_index)
        oasysgui.lineEdit(self.calculation_mode_1, self, "attenuation_coefficient", "Attenuation coefficient [cm-1]", labelWidth=260, valueType=float,
                           orientation="horizontal", callback=self.transfocator.dump_attenuation_coefficient)

        self.calculation_mode_2 = oasysgui.widgetBox(crl_box, "", addSpace=False, orientation="vertical")
        file_box = oasysgui.widgetBox(self.calculation_mode_2, "", addSpace=False, orientation="horizontal", height=20)
        self.le_prerefl_file = oasysgui.lineEdit(file_box, self, "prerefl_file", "File Prerefl", tooltip="prerefl_file[i]",
                                                 labelWidth=100, valueType=str, orientation="horizontal",
                                                 callback=self.transfocator.dump_prerefl_file)

        self.calculation_mode_3 = oasysgui.widgetBox(crl_box, "", addSpace=False, orientation="vertical")
        mat_box = oasysgui.widgetBox(self.calculation_mode_3, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(mat_box, self, "material", "Lens material", tooltip="material[i]",
                          labelWidth=90, valueType=str, orientation="horizontal",
                          callback=self.transfocator.dump_material)
        self.le_density = oasysgui.lineEdit(mat_box, self, "density", "density [g/cm3]", tooltip="density[i]",
                        labelWidth=110, valueType=float, orientation="horizontal",
                        callback=self.transfocator.dump_density)

        gui.button(file_box, self, "...", callback=self.selectFilePrerefl)
        self.set_ri_calculation_mode()

        ###############

        lens_box = tabs

        diameter_box_outer = oasysgui.widgetBox(lens_box, "", addSpace=False, orientation="horizontal")

        gui.comboBox(diameter_box_outer, self, "has_finite_diameter", label="Lens aperture", tooltip="has_finite_diameter[i]",
                     labelWidth=110, #labelWidth=260,
                     items=["Infinite", "Circular", "Square"], sendSelectedValue=False, orientation="horizontal", callback=self.set_diameter)

        self.diameter_box = oasysgui.widgetBox(diameter_box_outer, "", addSpace=False, orientation="vertical")
        self.diameter_box_empty = oasysgui.widgetBox(diameter_box_outer, "", addSpace=False, orientation="vertical", height=20)

        self.le_diameter = oasysgui.lineEdit(self.diameter_box, self, "diameter", " Value [\u03bcm]", tooltip="diameter[i]",
                                             labelWidth=80, #labelWidth=260,
                                             valueType=float, orientation="horizontal", callback=self.transfocator.dump_diameter)

        self.set_diameter()

        surface_shape_box_outer = oasysgui.widgetBox(lens_box, "", addSpace=False, orientation="horizontal")

        gui.comboBox(surface_shape_box_outer, self, "surface_shape", label="Surface Shape", #labelWidth=260,
                     tooltip="surface_shape[i]",
                     items=[ "Plane", "Sphere", "Paraboloid"], sendSelectedValue=False, orientation="horizontal",
                     callback=self.set_surface_shape)

        self.surface_shape_box = oasysgui.widgetBox(surface_shape_box_outer, "", addSpace=False, orientation="vertical")
        self.surface_shape_box_empty = oasysgui.widgetBox(surface_shape_box_outer, "", addSpace=False, orientation="vertical")

        self.le_radius = oasysgui.lineEdit(self.surface_shape_box, self, "radius", " Radius [\u03bcm]", tooltip="radius[i]",
                                           labelWidth=80, #labelWidth=260,
                                           valueType=float, orientation="horizontal", callback=self.transfocator.dump_radius)

        self.set_surface_shape()

        self.le_thickness = oasysgui.lineEdit(lens_box, self, "thickness", "Lens Thickness [\u03bcm]", labelWidth=260,
                                                   valueType=float, orientation="horizontal", tooltip="thickness[i]",
                                                   callback=self.transfocator.dump_thickness)

        self.le_piling_thickness = oasysgui.lineEdit(lens_box, self, "piling_thickness", "Piling thickness [mm]", labelWidth=260,
                                              valueType=float, orientation="horizontal", tooltip="piling_thickness[i]",
                                              callback=self.transfocator.dump_piling_thickness)

        gui.comboBox(oasysgui.widgetBox(lens_box, "", addSpace=False, orientation="vertical", height=40),
                     self, "convex_to_the_beam", label="1st interface exposed to the beam",
                            tooltip="convex_to_the_beam[i]", labelWidth=310,
                     items=["Concave", "Convex"], sendSelectedValue=False, orientation="horizontal", callback=self.transfocator.dump_convex_to_the_beam)


        gui.comboBox(lens_box, self, "is_cylinder", label="Cylindrical", tooltip="is_cylinder[i]", labelWidth=310,
                     items=["No", "Yes"], sendSelectedValue=False, orientation="horizontal", callback=self.set_cylindrical)

        self.box_cyl = oasysgui.widgetBox(lens_box, "", addSpace=False, orientation="vertical")
        self.box_cyl_empty = oasysgui.widgetBox(lens_box, "", addSpace=False, orientation="vertical", height=20)

        gui.comboBox(self.box_cyl, self, "cylinder_angle", tooltip="cylinder_angle[i]", label="Cylinder Angle (deg)",
                     labelWidth=260, items=["0 (Meridional)", "90 (Sagittal)"], sendSelectedValue=False,
                     orientation="horizontal", callback=self.transfocator.dump_cylinder_angle)

        self.set_cylindrical()

        self.is_on_init = False


    ############################################################
    #
    # GRAPHIC USER INTERFACE MANAGEMENT
    #
    ############################################################

    def selectFilePrerefl(self):
        self.le_prerefl_file.setText(oasysgui.selectFileFromDialog(self, self.prerefl_file, "Select File Prerefl", file_extension_filter="Data Files (*.dat)"))

        self.prerefl_file = self.le_prerefl_file.text()
        self.transfocator.dump_prerefl_file()

    def get_cylinder_angle(self):
        if self.is_cylinder:
            if self.cylinder_angle == 0:
                return 0.0
            elif self.cylinder_angle == 1:
                return 90.0
            else:
                raise ValueError("Cylinder Angle")
        else:
            return None

    def get_diameter(self):
        if self.has_finite_diameter > 0:
            return self.diameter
        else:
            return None

    def get_prerefl_file(self):
        if self.ri_calculation_mode == 1:
            return congruence.checkFileName(self.prerefl_file)
        else:
            return None

    def set_surface_shape(self):
        self.surface_shape_box.setVisible(self.surface_shape != 0)
        self.surface_shape_box_empty.setVisible(self.surface_shape == 0)

        if not self.is_on_init: self.transfocator.dump_surface_shape()

    def set_diameter(self):
        self.diameter_box.setVisible(self.has_finite_diameter > 0)
        self.diameter_box_empty.setVisible(self.has_finite_diameter == 0)

        if not self.is_on_init: self.transfocator.dump_has_finite_diameter()

    def set_cylindrical(self):
        self.box_cyl.setVisible(self.is_cylinder == 1)
        self.box_cyl_empty.setVisible(self.is_cylinder == 0)
        if not self.is_on_init: self.transfocator.dump_is_cylinder()

    def set_ri_calculation_mode(self):
        self.calculation_mode_1.setVisible(self.ri_calculation_mode == 0)
        self.calculation_mode_2.setVisible(self.ri_calculation_mode == 1)
        self.calculation_mode_3.setVisible(self.ri_calculation_mode > 1)

        if not self.is_on_init: self.transfocator.dump_ri_calculation_mode()

    def checkFields(self):
        congruence.checkPositiveNumber(self.n_lenses, "Number of lenses")
        congruence.checkPositiveNumber(self.slots_empty, "Number of empty slots")
        congruence.checkPositiveNumber(self.piling_thickness, "Piling thickness")

        congruence.checkNumber(self.p, "P")
        congruence.checkNumber(self.q, "Q")

        if self.has_finite_diameter == 0:
            congruence.checkStrictlyPositiveNumber(self.diameter, "Diameter")

        if self.ri_calculation_mode == 0:
            congruence.checkPositiveNumber(self.refraction_index, "Refraction Index")
            congruence.checkPositiveNumber(self.attenuation_coefficient, "Attenuation Coefficient")
        elif self.ri_calculation_mode == 1:
            congruence.checkFile(self.prerefl_file)
        else: # todo: check material
            congruence.checkPositiveNumber(self.density, "Density [g/cm3]")

        congruence.checkStrictlyPositiveNumber(self.radius, "Radius")
        congruence.checkPositiveNumber(self.thickness, "Lens Thickness")

    def setupUI(self):
        self.set_surface_shape()
        self.set_diameter()
        self.set_cylindrical()
        self.set_ri_calculation_mode()

if __name__ == "__main__":
    from shadow4.beamline.s4_beamline import S4Beamline
    import sys
    from orangecontrib.shadow4.util.shadow4_objects import ShadowData, PreReflPreProcessorData, VlsPgmPreProcessorData

    def get_test_beam():
        # electron beam
        from syned.storage_ring.light_source import ElectronBeam
        electron_beam = ElectronBeam(energy_in_GeV=6, energy_spread=0.001, current=0.2)
        electron_beam.set_sigmas_all(sigma_x=3.01836e-05, sigma_y=3.63641e-06, sigma_xp=4.36821e-06,
                                     sigma_yp=1.37498e-06)

        # Gaussian undulator
        from shadow4.sources.undulator.s4_undulator_gaussian import S4UndulatorGaussian
        sourceundulator = S4UndulatorGaussian(
            period_length=0.0159999,
            number_of_periods=100,
            photon_energy=2700.136,
            delta_e=0.0,
            flag_emittance=1,  # Use emittance (0=No, 1=Yes)
        )
        sourceundulator.set_energy_monochromatic(2700.14)

        from shadow4.sources.undulator.s4_undulator_gaussian_light_source import S4UndulatorGaussianLightSource
        light_source = S4UndulatorGaussianLightSource(name='GaussianUndulator', electron_beam=electron_beam,
                                              magnetic_structure=sourceundulator, nrays=5000, seed=5676561)

        beam = light_source.get_beam()

        return ShadowData(beam=beam, beamline=S4Beamline(light_source=light_source))

    from PyQt5.QtWidgets import QApplication
    a = QApplication(sys.argv)
    ow = OWTransfocator()
    ow.view_type = 2
    ow.set_shadow_data(get_test_beam())

    ow.show()
    a.exec_()
    ow.saveSettings()


import os, sys
import numpy

from PyQt5.QtWidgets import QLabel, QApplication, QMessageBox, QSizePolicy
from PyQt5.QtGui import QTextCursor, QPixmap, QDoubleValidator
from PyQt5.QtCore import Qt

from shadow4.physical_models.prerefl.prerefl import PreRefl

import orangecanvas.resources as resources

from orangewidget import gui, widget
from orangewidget.settings import Setting

from oasys.widgets.widget import OWWidget
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence

from oasys.util.oasys_util import EmittingStream
from orangecontrib.shadow4.util.shadow4_objects import PreReflPreProcessorData
from orangecontrib.shadow4.util.shadow4_util import ShadowPhysics
from orangecontrib.shadow4.widgets.gui.plots import plot_data1D, plot_data2D, plot_multi_data1D


class OWPrerefl(OWWidget):
    name = "PreRefl"
    id = "xsh_prerefl"
    description = "Calculation of mirror reflectivity profile"
    icon = "icons/prerefl.png"
    author = "create_widget.py"
    maintainer_email = "srio@esrf.eu"
    priority = 10
    category = ""
    keywords = ["xoppy", "xsh_prerefl"]

    outputs = [{"name":"PreReflPreProcessorData",
                "type":PreReflPreProcessorData,
                "doc":"PreRefl PreProcessor Data",
                "id":"PreReflPreProcessorData"}]

    want_main_area = True

    symbol = Setting("SiC")
    density = Setting(3.217)
    prerefl_file = Setting("reflec.dat")
    e_min = Setting(100.0)
    e_max = Setting(20000.0)
    e_step = Setting(100.0)

    #
    # Plots
    #
    plot_flag = Setting(0)
    scan_e_n = Setting(100)
    scan_e_from = Setting(5000.0)
    scan_e_to = Setting(10000.0)
    scan_a0 = Setting(5.0)

    scan_a_n = Setting(100)
    scan_a_from = Setting(0)
    scan_a_to = Setting(10.0)
    scan_e0 = Setting(8000.0)


    IMAGE_WIDTH  = 860
    IMAGE_HEIGHT = 545

    MAX_WIDTH          = 1320
    MAX_HEIGHT         = 700
    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT   = 630

    usage_path = os.path.join(resources.package_dirname("orangecontrib.shadow4.widgets.gui") , "misc", "prerefl_usage.png")

    prerefl_instance = None

    def __init__(self):
        super().__init__()

        self.runaction = widget.OWAction("Compute", self)
        self.runaction.triggered.connect(self.compute)
        self.addAction(self.runaction)

        self.setFixedWidth(self.MAX_WIDTH)
        self.setFixedHeight(self.MAX_HEIGHT)

        gui.separator(self.controlArea)

        box0 = gui.widgetBox(self.controlArea, "",orientation="horizontal")

        button = gui.button(box0, self, "Compute", callback=self.compute)
        button.setFixedHeight(35)

        gui.separator(self.controlArea)

        tabs_setting = oasysgui.tabWidget(self.controlArea)
        tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT)
        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)

        self.main_tabs = oasysgui.tabWidget(self.mainArea)
        tab_out = oasysgui.createTabPage(self.main_tabs, "Output")
        self.plot_tab = oasysgui.createTabPage(self.main_tabs, "Plots (optional scan)")

        tab_bas = oasysgui.createTabPage(tabs_setting, "Reflectivity Settings")

        tab_input_plots = oasysgui.createTabPage(tabs_setting, "Plots")

        tab_usa = oasysgui.createTabPage(tabs_setting, "Use of the Widget")

        self.populate_tab_basic_settings(tab_bas)

        self.populate_tab_plots(tab_input_plots)

        self.populate_tab_use_of_widget(tab_usa)

        self.shadow_output = oasysgui.textArea()
        out_box = oasysgui.widgetBox(tab_out, "System Output", addSpace=True, orientation="horizontal", height=self.TABS_AREA_HEIGHT)
        out_box.layout().addWidget(self.shadow_output)

        self.process_showers()

        gui.rubber(self.controlArea)

        self.set_visibility()

    def populate_tab_basic_settings(self, tab_bas):
        box = oasysgui.widgetBox(tab_bas, "Reflectivity Parameters", orientation="vertical")
        idx = -1

        # widget index 0
        idx += 1
        oasysgui.lineEdit(box, self, "symbol", tooltip="symbol",
                          label=self.unitLabels()[idx], addSpace=True, labelWidth=200, orientation="horizontal",
                          callback=self.set_Density)
        self.show_at(self.unitFlags()[idx], box)

        # widget index 1
        idx += 1
        oasysgui.lineEdit(box, self, "density", tooltip="density",
                          label=self.unitLabels()[idx], addSpace=True, valueType=float, labelWidth=200,
                          orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

        # widget index 2
        idx += 1
        box_2 = oasysgui.widgetBox(box, "", addSpace=True, orientation="horizontal")

        self.le_prerefl_file = oasysgui.lineEdit(box_2, self, "prerefl_file", tooltip="prerefl_file",
                                                 label=self.unitLabels()[idx], addSpace=True, labelWidth=180,
                                                 orientation="horizontal")

        gui.button(box_2, self, "...", callback=self.selectFile)

        self.show_at(self.unitFlags()[idx], box)

        # widget index 3
        idx += 1
        oasysgui.lineEdit(box, self, "e_min", tooltip="e_min",
                          label=self.unitLabels()[idx], addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

        # widget index 4
        idx += 1
        oasysgui.lineEdit(box, self, "e_max", tooltip="e_max",
                          label=self.unitLabels()[idx], addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

        # widget index 5
        idx += 1
        oasysgui.lineEdit(box, self, "e_step", tooltip="e_step",
                          label=self.unitLabels()[idx], addSpace=True,
                          valueType=float, labelWidth=200, orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box)

    def populate_tab_plots(self, tab_plots):

        box = gui.widgetBox(tab_plots, "Optional scan plots", orientation="vertical")

        gui.comboBox(box, self, "plot_flag", tooltip="plot_flag",
                     label="Scan plot", addSpace=True,
                     items=['No', 'refraction index', 'attenuation coefficient',
                            'mirror reflectivity energy-scan',
                            'mirror reflectivity grazing angle-scan',
                            'mirror reflectivity energy-angle-scan'],
                     valueType=int, orientation="horizontal", labelWidth=270, callback=self.do_plots)

        self.box_plots = gui.widgetBox(tab_plots, "Optional scan plots", orientation="vertical")

        self.box_plot_e = oasysgui.widgetBox(self.box_plots, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(self.box_plot_e, self, "scan_e_from", label="from [eV]", tooltip="scan_e_from",
                          valueType=float, addSpace=True, labelWidth=100, orientation="horizontal",
                          callback=self.do_plots)
        oasysgui.lineEdit(self.box_plot_e, self, "scan_e_to", label="to [eV]", tooltip="scan_e_to",
                          valueType=float, addSpace=True, labelWidth=20, orientation="horizontal",
                          callback=self.do_plots)
        oasysgui.lineEdit(self.box_plot_e, self, "scan_e_n", label="points", tooltip="scan_e_n",
                          valueType=int, addSpace=True, labelWidth=50, orientation="horizontal",
                          callback=self.do_plots)
        self.box_plot_a0 = oasysgui.widgetBox(self.box_plots, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(self.box_plot_a0, self, "scan_a0", label="Fixed angle [mdeg]", tooltip="scan_a0",
                          valueType=float, addSpace=True, labelWidth=200, orientation="horizontal",
                          callback=self.do_plots)

        self.box_plot_a = oasysgui.widgetBox(self.box_plots, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(self.box_plot_a, self, "scan_a_from", label="from [deg]", tooltip="scan_a_from",
                          valueType=float, addSpace=True, labelWidth=100, orientation="horizontal",
                          callback=self.do_plots)
        oasysgui.lineEdit(self.box_plot_a, self, "scan_a_to", label="to [deg]", tooltip="scan_a_to",
                          valueType=float, addSpace=True, labelWidth=20, orientation="horizontal",
                          callback=self.do_plots)
        oasysgui.lineEdit(self.box_plot_a, self, "scan_a_n", label="points", tooltip="scan_a_n",
                          valueType=int, addSpace=True, labelWidth=50, orientation="horizontal",
                          callback=self.do_plots)
        self.box_plot_e0 = oasysgui.widgetBox(self.box_plots, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(self.box_plot_e0, self, "scan_e0", label="Fixed energy [eV]", tooltip="scan_e0",
                          valueType=float, addSpace=True, labelWidth=200, orientation="horizontal",
                          callback=self.do_plots)


    def populate_tab_use_of_widget(self, tab_usa):
        tab_usa.setStyleSheet("background-color: white;")
        usage_box = oasysgui.widgetBox(tab_usa, "", addSpace=True, orientation="horizontal")
        label = QLabel("")
        label.setAlignment(Qt.AlignCenter)
        label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        label.setPixmap(QPixmap(self.usage_path))
        usage_box.layout().addWidget(label)

    def set_visibility(self):
        self.box_plots.setVisible(self.plot_flag > 0)
        self.box_plot_e.setVisible(False)
        self.box_plot_a0.setVisible(False)
        self.box_plot_a.setVisible(False)
        self.box_plot_e0.setVisible(False)

        if self.plot_flag == 1:
            self.box_plot_e.setVisible(True)
        elif self.plot_flag == 2:
            self.box_plot_e.setVisible(True)
        elif self.plot_flag == 3:
            self.box_plot_e.setVisible(True)
            self.box_plot_a0.setVisible(True)
        elif self.plot_flag == 4:
            self.box_plot_a.setVisible(True)
            self.box_plot_e0.setVisible(True)
        elif self.plot_flag == 5:
            self.box_plot_a.setVisible(True)
            self.box_plot_e.setVisible(True)

    def unitLabels(self):
         return ['Element/Compound formula','Density [ g/cm3 ]','File name (for SHADOW):','Minimum energy [eV]','Maximum energy [eV]','Energy step [eV]']

    def unitFlags(self):
         return ['True','True','True','True','True','True']

    def selectFile(self):
        self.le_prerefl_file.setText(oasysgui.selectFileFromDialog(self, self.prerefl_file, "Select Output File", file_extension_filter="Data Files (*.dat)"))

    def set_Density(self):
        if not self.symbol is None:
            if not self.symbol.strip() == "":
                self.symbol = self.symbol.strip()
                self.density = ShadowPhysics.getMaterialDensity(self.symbol)


    def compute(self):
        try:
            sys.stdout = EmittingStream(textWritten=self.writeStdOut)

            self.checkFields()

            PreRefl.prerefl(interactive=False,
                            SYMBOL=self.symbol,
                            DENSITY=self.density,
                            FILE=congruence.checkFileName(self.prerefl_file),
                            E_MIN=self.e_min,
                            E_MAX=self.e_max,
                            E_STEP=self.e_step)

            self.send("PreReflPreProcessorData", PreReflPreProcessorData(prerefl_data_file=self.prerefl_file))

            self.prerefl_instance = PreRefl()
            self.prerefl_instance.read_preprocessor_file(self.prerefl_file)
            self.prerefl_instance.preprocessor_info()
            self.do_plots()
        except Exception as exception:
            QMessageBox.critical(self, "Error",
                                 str(exception),
                                 QMessageBox.Ok)

    def checkFields(self):
        self.symbol = ShadowPhysics.checkCompoundName(self.symbol)
        self.density = congruence.checkStrictlyPositiveNumber(self.density, "Density")
        self.e_min  = congruence.checkPositiveNumber(self.e_min , "Minimum Energy")
        self.e_max  = congruence.checkStrictlyPositiveNumber(self.e_max , "Maximum Energy")
        self.e_step = congruence.checkStrictlyPositiveNumber(self.e_step, "Energy step")
        congruence.checkLessOrEqualThan(self.e_min, self.e_max, "Minimum Energy", "Maximum Energy")
        congruence.checkDir(self.prerefl_file)

    def do_plots(self):

        self.set_visibility()

        self.plot_tab.layout().removeItem(self.plot_tab.layout().itemAt(0))

        if self.prerefl_instance is None: return

        if self.plot_flag == 0:
            plot_widget_id = plot_data1D([0], [0], xtitle="", ytitle="")
        elif self.plot_flag == 1:
            energy_array = numpy.linspace(self.scan_e_from, self.scan_e_to, self.scan_e_n)
            refraction_index = self.prerefl_instance.get_refraction_index(energy_array)
            delta = 1.0 - refraction_index.real
            beta = refraction_index.imag

            plot_widget_id = plot_multi_data1D(energy_array, [delta, beta],
                                         xtitle="Photon energy [eV]",
                                         ytitle="(n = 1 - delta + i beta)",
                                         ytitles=['delta', 'beta'])
        elif self.plot_flag == 2:
            energy_array = numpy.linspace(self.scan_e_from, self.scan_e_to, self.scan_e_n)
            att = self.prerefl_instance.get_attenuation_coefficient(energy_array)

            plot_widget_id = plot_data1D(energy_array, att,
                                         xtitle="Photon energy [eV]",
                                         ytitle="mu - attenuation coefficient cm^-1",
                                         )
        elif self.plot_flag == 3:
            energy_array = numpy.linspace(self.scan_e_from, self.scan_e_to, self.scan_e_n)
            RS, RP, _ = self.prerefl_instance.reflectivity_fresnel(grazing_angle_mrad=self.scan_a0,
                                                                           photon_energy_ev=energy_array,
                                                                           roughness_rms_A=0.0,
                                                                           )

            plot_widget_id = plot_multi_data1D(energy_array, [RS, RP],
                                         xtitle="Photon energy [eV]",
                                         ytitle="Mirror reflectivity @ %.3f mrad" % self.scan_a0,
                                         ytitles=['S-polarized', 'P-polarized'])
        elif self.plot_flag == 4:
            angle_array = numpy.linspace(self.scan_a_from, self.scan_a_to, self.scan_a_n)
            RS, RP, _ = self.prerefl_instance.reflectivity_fresnel(grazing_angle_mrad=angle_array,
                                                                           photon_energy_ev=self.scan_e0,
                                                                           roughness_rms_A=0.0,
                                                                           )

            plot_widget_id = plot_multi_data1D(angle_array, [RS, RP],
                                         xtitle="Grazing angle [mrad]",
                                         ytitle="Mirror reflectivity @ %.3f eV" % self.scan_e0,
                                         ytitles=['S-polarized', 'P-polarized'])

        elif self.plot_flag == 5:
            angle_array = numpy.linspace(self.scan_a_from, self.scan_a_to, self.scan_a_n)
            energy_array = numpy.linspace(self.scan_e_from, self.scan_e_to, self.scan_e_n)
            E = numpy.outer(energy_array, numpy.ones_like(angle_array))
            A = numpy.outer(numpy.ones_like(energy_array), angle_array)
            RS, RP, _ = self.prerefl_instance.reflectivity_fresnel(grazing_angle_mrad=A,
                                                                   photon_energy_ev=E,
                                                                   roughness_rms_A=0.0,
                                                                   )
            RS.shape = (energy_array.size, angle_array.size)
            # R_S_array = RS.reshape((energyN, thetaN))
            # A.shape = (energy_array.size, angle_array.shape)

            plot_widget_id = plot_data2D(RS, energy_array, angle_array,
                                         xtitle="Photon energy [eV]",
                                         ytitle="Grazing angle [mrad]",
                                         title="Mirror reflectivity (S-pol)",
                                         )

            # energyN = 1
            # energy1 = self.scan_e0
            # energy2 = self.scan_e0
            # thetaN = int(self.scan_a_n)
            # theta1 = self.scan_a_from
            # theta2 = self.scan_a_to
            # R_S_array, R_P_array, energy_array, theta_array = self.mlayer_instance.scan(
            #     energyN=energyN, energy1=energy1, energy2=energy2,
            #     thetaN=thetaN, theta1=theta1, theta2=theta2)
            # plot_widget_id = plot_data1D(theta_array, R_S_array[0, :]**2, xtitle="grazing angle [deg]", ytitle="Reflectivity")
        # elif self.plot_flag == 3:
        #     energyN = self.scan_e_n
        #     energy1 = self.scan_e_from
        #     energy2 = self.scan_e_to
        #     thetaN = self.scan_a_n
        #     theta1 = self.scan_a_from
        #     theta2 = self.scan_a_to
        #     R_S_array, R_P_array, energy_array, theta_array = self.mlayer_instance.scan(
        #         energyN=energyN, energy1=energy1, energy2=energy2,
        #         thetaN=thetaN, theta1=theta1, theta2=theta2)
        #     plot_widget_id = plot_data2D(R_S_array**2, energy_array, theta_array, title="title",
        #                                  xtitle="photon energy [eV]", ytitle="grazing angle [deg]")

        self.plot_tab.layout().addWidget(plot_widget_id)

    def writeStdOut(self, text):
        cursor = self.shadow_output.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.insertText(text)
        self.shadow_output.setTextCursor(cursor)
        self.shadow_output.ensureCursorVisible()

if __name__ == "__main__":
    import sys
    app = QApplication(sys.argv)
    w = OWPrerefl()
    w.show()
    app.exec()
    w.saveSettings()
